@@ -24,6 +24,7 @@
 
 #include <fastdds/dds/core/policy/ParameterTypes.hpp>
 
+#include <fastdds/rtps/attributes/ExternalLocators.hpp>
 #include <fastdds/rtps/attributes/PropertyPolicy.h>
 #include <fastdds/rtps/attributes/RTPSParticipantAllocationAttributes.hpp>
 #include <fastdds/rtps/attributes/RTPSParticipantAttributes.h>
@@ -119,6 +120,7 @@ public:
 
     /**
      * @brief Constructor
+     *
      * @param send_always Boolean that set if the Qos need to be sent even if it is not changed
      */
     explicit QosPolicy(
@@ -130,6 +132,7 @@ public:
 
     /**
      * @brief Copy Constructor
+     *
      * @param b Another instance of QosPolicy
      */
     QosPolicy(
@@ -152,6 +155,7 @@ public:
 
     /**
      * Whether it should always be sent.
+     *
      * @return True if it should always be sent.
      */
     virtual bool send_always() const
@@ -173,6 +177,7 @@ protected:
 /**
  * @brief Controls the behavior of the entity when acting as a factory for other entities. In other words,
  * configures the side-effects of the create_* and delete_* operations.
+ *
  * @note Mutable Qos Policy
  */
 class EntityFactoryQosPolicy
@@ -196,6 +201,7 @@ public:
 
     /**
      * @brief Constructor
+     *
      * @param autoenable Value for the autoenable_created_entities boolean
      */
     RTPS_DllAPI EntityFactoryQosPolicy(
@@ -249,6 +255,7 @@ typedef enum DurabilityQosPolicyKind : fastrtps::rtps::octet
     TRANSIENT_DURABILITY_QOS,
     /**
      * Data is kept on permanent storage, so that they can outlive a system session.
+     *
      * @warning Not Supported
      */
     PERSISTENT_DURABILITY_QOS
@@ -259,6 +266,7 @@ typedef enum DurabilityQosPolicyKind : fastrtps::rtps::octet
 
 /**
  * This policy expresses if the data should ‘outlive’ their writing time.
+ *
  * @note Immutable Qos Policy
  */
 class DurabilityQosPolicy : public Parameter_t, public QosPolicy
@@ -282,6 +290,7 @@ public:
 
     /**
      * Translates kind to rtps layer equivalent
+     *
      * @return fastrtps::rtps::DurabilityKind_t
      */
     inline fastrtps::rtps::DurabilityKind_t durabilityKind() const
@@ -306,6 +315,7 @@ public:
 
     /**
      * Set kind passing the rtps layer equivalent kind
+     *
      * @param new_kind fastrtps::rtps::DurabilityKind_t
      */
     inline void durabilityKind(
@@ -341,6 +351,7 @@ public:
  * @brief DataReader expects a new sample updating the value of each instance at least once every deadline period.
  * DataWriter indicates that the application commits to write a new value (using the DataWriter) for each instance managed
  * by the DataWriter at least once every deadline period.
+ *
  * @note Mutable Qos Policy
  */
 class DeadlineQosPolicy : public Parameter_t, public QosPolicy
@@ -391,7 +402,9 @@ public:
  * Specifies the maximum acceptable delay from the time the data is written until the data is inserted in the receiver's
  * application-cache and the receiving application is notified of the fact.This policy is a hint to the Service, not something
  * that must be monitored or enforced. The Service is not required to track or alert the user of any violation.
+ *
  * @warning This QosPolicy can be defined and is transmitted to the rest of the network but is not implemented in this version.
+ *
  * @note Mutable Qos Policy
  */
 class LatencyBudgetQosPolicy : public Parameter_t, public QosPolicy
@@ -507,16 +520,19 @@ public:
 
     //! Liveliness kind <br> By default, AUTOMATIC_LIVELINESS.
     LivelinessQosPolicyKind kind;
-    //! Period within which liveliness should be asserted.
-    //! On a DataWriter it represents the period it commits to signal its liveliness.
-    //! On a DataReader it represents the period without assertion after which a DataWriter is considered
-    //! inactive.
-    //! By default, c_TimeInfinite.
+    /*! Period within which liveliness should be asserted.
+     *  On a DataWriter it represents the period it commits to signal its liveliness.
+     *  On a DataReader it represents the period without assertion after which a DataWriter is considered
+     *  inactive.
+     *  By default, c_TimeInfinite.
+     */
     fastrtps::Duration_t lease_duration;
-    //! The period for automatic assertion of liveliness.
-    //! Only used for DataWriters with AUTOMATIC liveliness.
-    //! By default, c_TimeInfinite.
-    //! @warning When not infinite, must be < lease_duration, and it is advisable to be less than 0.7*lease_duration.
+    /*! The period for automatic assertion of liveliness.
+     *  Only used for DataWriters with AUTOMATIC liveliness.
+     *  By default, c_TimeInfinite.
+     *
+     * @warning When not infinite, must be < lease_duration, and it is advisable to be less than 0.7*lease_duration.
+     */
     fastrtps::Duration_t announcement_period;
 };
 
@@ -542,6 +558,7 @@ typedef enum ReliabilityQosPolicyKind : fastrtps::rtps::octet
 
 /**
  * Indicates the reliability of the endpoint.
+ *
  * @note Immutable Qos Policy
  */
 class ReliabilityQosPolicy : public Parameter_t, public QosPolicy
@@ -623,6 +640,7 @@ enum OwnershipQosPolicyKind : fastrtps::rtps::octet
 /**
  * Specifies whether it is allowed for multiple DataWriters to write the same instance of the data and if so, how these
  * modifications should be arbitrated
+ *
  * @note Immutable Qos Policy
  */
 class OwnershipQosPolicy : public Parameter_t, public QosPolicy
@@ -687,7 +705,9 @@ enum DestinationOrderQosPolicyKind : fastrtps::rtps::octet
 /**
  * Controls the criteria used to determine the logical order among changes made by Publisher entities to the same instance of
  * data (i.e., matching Topic and key).
+ *
  * @warning This QosPolicy can be defined and is transmitted to the rest of the network but is not implemented in this version.
+ *
  * @note Immutable Qos Policy
  */
 class DestinationOrderQosPolicy : public Parameter_t, public QosPolicy
@@ -856,6 +876,7 @@ public:
 
     /**
      * Set the maximum size of the user data and reserves memory for that much.
+     *
      * @param size new maximum size of the user data. Zero for unlimited size
      */
     void set_max_size (
@@ -894,6 +915,7 @@ public:
 
     /**
      * Returns raw data vector.
+     *
      * @return raw data as vector of octets.
      * */
     RTPS_DllAPI inline const collection_type& data_vec() const
@@ -903,6 +925,7 @@ public:
 
     /**
      * Returns raw data vector.
+     *
      * @return raw data as vector of octets.
      * */
     RTPS_DllAPI inline collection_type& data_vec()
@@ -912,6 +935,7 @@ public:
 
     /**
      * Sets raw data vector.
+     *
      * @param vec raw data to set.
      * */
     RTPS_DllAPI inline void data_vec(
@@ -927,6 +951,7 @@ public:
 
     /**
      * Returns raw data vector.
+     *
      * @return raw data as vector of octets.
      * */
     RTPS_DllAPI inline const collection_type& getValue() const
@@ -936,6 +961,7 @@ public:
 
     /**
      * Sets raw data vector.
+     *
      * @param vec raw data to set.
      * */
     RTPS_DllAPI inline void setValue(
@@ -950,6 +976,7 @@ public:
  * Class TClassName, base template for data qos policies.
  * Data not known by the middleware, but distributed by means of built-in topics.
  * By default, zero-sized sequence.
+ *
  * @note Mutable Qos Policy
  */
 // *INDENT-OFF*  (uncrustify seems to have problems with this macro)
@@ -1063,7 +1090,9 @@ TEMPLATE_DATA_QOS_POLICY(GroupDataQosPolicy, PID_GROUP_DATA)
  * The filter states that the DataReader does not want to receive more than one value each minimum_separation, regardless
  * of how fast the changes occur. It is inconsistent for a DataReader to have a minimum_separation longer than its
  * Deadline period.
+ *
  * @warning This QosPolicy can be defined and is transmitted to the rest of the network but is not implemented in this version.
+ *
  * @note Mutable Qos Policy
  */
 class TimeBasedFilterQosPolicy : public Parameter_t, public QosPolicy
@@ -1135,7 +1164,9 @@ enum PresentationQosPolicyAccessScopeKind : fastrtps::rtps::octet
  * order of changes.access_scope determines the largest scope spanning the entities for which the order and coherency
  * of changes can be preserved. The two booleans control whether coherent access and ordered access are supported within
  * the scope access_scope.
+ *
  * @warning This QosPolicy can be defined and is transmitted to the rest of the network but is not implemented in this version.
+ *
  * @note Immutable Qos Policy
  */
 class PresentationQosPolicy : public Parameter_t, public QosPolicy
@@ -1214,6 +1245,7 @@ public:
 
     /**
      * @brief Constructor using a pointer
+     *
      * @param ptr Pointer to be set
      */
     explicit Partition_t(
@@ -1237,6 +1269,7 @@ public:
 
     /**
      * @brief Getter for the size
+     *
      * @return uint32_t with the size
      */
     uint32_t size() const
@@ -1246,6 +1279,7 @@ public:
 
     /**
      * @brief Getter for the partition name
+     *
      * @return name
      */
     const char* name() const
@@ -1262,6 +1296,7 @@ public:
  *
  * The empty string ("") is considered a valid partition that is matched with other partition names using the same rules of
  * string matching and regular-expression matching used for any other partition name.
+ *
  * @note Mutable Qos Policy
  */
 class PartitionQosPolicy : public Parameter_t, public QosPolicy
@@ -1281,6 +1316,7 @@ public:
 
         /**
          * @brief Constructor using a pointer
+         *
          * @param ptr Pointer to be set
          */
         const_iterator(
@@ -1363,6 +1399,7 @@ public:
 
     /**
      * @brief Constructor using Parameter length
+     *
      * @param in_length Length of the parameter
      */
     RTPS_DllAPI PartitionQosPolicy(
@@ -1377,6 +1414,7 @@ public:
 
     /**
      * @brief Copy constructor
+     *
      * @param b Another PartitionQosPolicy instance
      */
     RTPS_DllAPI PartitionQosPolicy(
@@ -1424,6 +1462,7 @@ public:
 
     /**
      * @brief Getter for the first position of the partition list
+     *
      * @return const_iterator
      */
     const_iterator begin() const
@@ -1433,6 +1472,7 @@ public:
 
     /**
      * @brief Getter for the end of the partition list
+     *
      * @return const_iterator
      */
     const_iterator end() const
@@ -1442,6 +1482,7 @@ public:
 
     /**
      * @brief Getter for the number of partitions
+     *
      * @return uint32_t with the size
      */
     uint32_t size() const
@@ -1451,6 +1492,7 @@ public:
 
     /**
      * @brief Check if the set is empty
+     *
      * @return true if it is empty, false otherwise
      */
     uint32_t empty() const
@@ -1460,6 +1502,7 @@ public:
 
     /**
      * @brief Setter for the maximum size reserved for partitions (in bytes)
+     *
      * @param size Size to be set
      */
     void set_max_size (
@@ -1471,6 +1514,7 @@ public:
 
     /**
      * @brief Getter for the maximum size (in bytes)
+     *
      * @return uint32_t with the maximum size
      */
     uint32_t max_size () const
@@ -1480,6 +1524,7 @@ public:
 
     /**
      * Appends a name to the list of partition names.
+     *
      * @param name Name to append.
      */
     RTPS_DllAPI inline void push_back(
@@ -1523,6 +1568,7 @@ public:
 
     /**
      * Returns partition names.
+     *
      * @return Vector of partition name strings.
      */
     RTPS_DllAPI inline const std::vector<std::string> getNames() const
@@ -1532,6 +1578,7 @@ public:
 
     /**
      * Overrides partition names
+     *
      * @param nam Vector of partition name strings.
      */
     RTPS_DllAPI inline void setNames(
@@ -1542,6 +1589,7 @@ public:
 
     /**
      * Returns partition names.
+     *
      * @return Vector of partition name strings.
      */
     RTPS_DllAPI inline const std::vector<std::string> names() const
@@ -1559,6 +1607,7 @@ public:
 
     /**
      * Overrides partition names
+     *
      * @param nam Vector of partition name strings.
      */
     RTPS_DllAPI inline void names(
@@ -1612,6 +1661,7 @@ enum HistoryQosPolicyKind : fastrtps::rtps::octet
  * existing DataReader entities. The behavior with regards to a DataReaderentities discovered after a sample is written
  * is controlled by the DURABILITY QoS policy. On the subscribing side it controls the samples that should be maintained
  * until the application “takes” them from the Service.
+ *
  * @note Immutable Qos Policy
  */
 class HistoryQosPolicy : public Parameter_t, public QosPolicy
@@ -1653,9 +1703,11 @@ public:
 
     //!HistoryQosPolicyKind. <br> By default, KEEP_LAST_HISTORY_QOS.
     HistoryQosPolicyKind kind;
-    //!History depth. <br> By default, 1. If a value other than 1 is specified, it should
-    //! be consistent with the settings of the ResourceLimitsQosPolicy.
-    //! @warning Only takes effect if the kind is KEEP_LAST_HISTORY_QOS.
+    /*! History depth. <br> By default, 1. If a value other than 1 is specified, it should
+     *  be consistent with the settings of the ResourceLimitsQosPolicy.
+     *
+     *  @warning Only takes effect if the kind is KEEP_LAST_HISTORY_QOS.
+     */
     int32_t depth;
 };
 
@@ -1664,6 +1716,7 @@ constexpr int32_t LENGTH_UNLIMITED = -1;
 
 /**
  * Specifies the resources that the Service can consume in order to meet the requested QoS
+ *
  * @note Immutable Qos Policy
  */
 class ResourceLimitsQosPolicy : public Parameter_t, public QosPolicy
@@ -1675,6 +1728,7 @@ public:
      * instances associated with it. Represents the maximum samples the middleware can store for any one DataWriter
      * (or DataReader). <br>
      * By default, 5000.
+     *
      * @warning It is inconsistent for this value to be less than max_samples_per_instance.
      */
     int32_t max_samples;
@@ -1686,6 +1740,7 @@ public:
     /**
      * @brief Represents the maximum number of samples of any one instance a DataWriter(or DataReader) can manage. <br>
      * By default, 400.
+     *
      * @warning It is inconsistent for this value to be greater than max_samples.
      */
     int32_t max_samples_per_instance;
@@ -1743,7 +1798,9 @@ public:
 /**
  * Specifies the configuration of the durability service. That is, the service that implements the DurabilityQosPolicy kind
  * of TRANSIENT and PERSISTENT.
+ *
  * @warning This QosPolicy can be defined and is transmitted to the rest of the network but is not implemented in this version.
+ *
  * @note Immutable Qos Policy
  */
 class DurabilityServiceQosPolicy : public Parameter_t, public QosPolicy
@@ -1831,6 +1888,7 @@ public:
 
 /**
  * Specifies the maximum duration of validity of the data written by the DataWriter.
+ *
  * @note Mutable Qos Policy
  */
 class LifespanQosPolicy : public Parameter_t, public QosPolicy
@@ -1876,6 +1934,7 @@ public:
  * Specifies the value of the “strength” used to arbitrate among multiple DataWriter objects that attempt to modify the same
  * instance of a data-object (identified by Topic + key).This policy only applies if the OWNERSHIP QoS policy is of kind
  * EXCLUSIVE.
+ *
  * @note Mutable Qos Policy
  */
 class OwnershipStrengthQosPolicy : public Parameter_t, public QosPolicy
@@ -1920,7 +1979,9 @@ public:
 
 /**
  * This policy is a hint to the infrastructure as to how to set the priority of the underlying transport used to send the data.
+ *
  * @warning This QosPolicy can be defined and is transmitted to the rest of the network but is not implemented in this version.
+ *
  * @note Mutable Qos Policy
  */
 class TransportPriorityQosPolicy : public Parameter_t, public QosPolicy
@@ -1980,8 +2041,10 @@ public:
     //!PublishModeQosPolicyKind <br> By default, SYNCHRONOUS_PUBLISH_MODE.
     PublishModeQosPolicyKind kind = SYNCHRONOUS_PUBLISH_MODE;
 
-    //! Name of the flow controller used when publish mode kind is ASYNCHRONOUS_PUBLISH_MODE.
-    //! @since 2.4.0
+    /*! Name of the flow controller used when publish mode kind is ASYNCHRONOUS_PUBLISH_MODE.
+     *
+     * @since 2.4.0
+     */
     const char* flow_controller_name = fastdds::rtps::FASTDDS_FLOW_CONTROLLER_DEFAULT;
 
     inline void clear() override
@@ -1990,6 +2053,14 @@ public:
         std::swap(*this, reset);
     }
 
+    bool operator ==(
+            const PublishModeQosPolicy& b) const
+    {
+        return (this->kind == b.kind) &&
+               0 == strcmp(flow_controller_name, b.flow_controller_name) &&
+               QosPolicy::operator ==(b);
+    }
+
 };
 
 /**
@@ -2006,8 +2077,10 @@ typedef enum DataRepresentationId : int16_t
  * With multiple standard data Representations available, and vendor-specific extensions possible, DataWriters and
  * DataReaders must be able to negotiate which data representation(s) to use. This negotiation shall occur based on
  * DataRepresentationQosPolicy.
+ *
  * @warning If a writer’s offered representation is contained within a reader’s sequence, the offer satisfies the
  * request and the policies are compatible. Otherwise, they are incompatible.
+ *
  * @note Immutable Qos Policy
  */
 class DataRepresentationQosPolicy : public Parameter_t, public QosPolicy
@@ -2033,6 +2106,7 @@ public:
 
     /**
      * Compares the given policy to check if it's equal.
+     *
      * @param b QoS Policy.
      * @return True if the policy is equal.
      */
@@ -2068,6 +2142,7 @@ enum TypeConsistencyKind : uint16_t
 /**
  * The TypeConsistencyEnforcementQosPolicy defines the rules for determining whether the type used to publish a given data
  * stream is consistent with that used to subscribe to it. It applies to DataReaders.
+ *
  * @note Immutable Qos Policy
  */
 class TypeConsistencyEnforcementQosPolicy : public Parameter_t, public QosPolicy
@@ -2087,7 +2162,7 @@ public:
     bool m_ignore_sequence_bounds;
     /**
      * @brief This option controls whether string bounds are taken into consideration for type assignability. If the option
-     *  is set to TRUE, string bounds (maximum lengths) are not considered as part of the type assignability. This means
+     * is set to TRUE, string bounds (maximum lengths) are not considered as part of the type assignability. This means
      * that a T2 string type with maximum length L2 would be assignable to a T1 string type with maximum length L1, even
      * if L2 is greater than L1. If the option is set to false, then string bounds are taken into consideration for type
      * assignability and in order for T1 to be assignable from T2 it is required that L1>= L2. <br>
@@ -2157,6 +2232,7 @@ public:
 
 /**
  * Class DisablePositiveACKsQosPolicy to disable sending of positive ACKs
+ *
  * @note Immutable Qos Policy
  */
 class DisablePositiveACKsQosPolicy : public Parameter_t, public QosPolicy
@@ -2224,6 +2300,7 @@ public:
 
     /**
      * @brief Copy constructor
+     *
      * @param type Another instance of TypeIdV1
      */
     RTPS_DllAPI TypeIdV1(
@@ -2236,6 +2313,7 @@ public:
 
     /**
      * @brief Constructor using a TypeIndentifier
+     *
      * @param identifier TypeIdentifier to be set
      */
     RTPS_DllAPI TypeIdV1(
@@ -2248,6 +2326,7 @@ public:
 
     /**
      * @brief Move constructor
+     *
      * @param type Another instance of TypeIdV1
      */
     RTPS_DllAPI TypeIdV1(
@@ -2301,6 +2380,7 @@ public:
 
     /**
      * @brief Getter for the TypeIndentifier
+     *
      * @return TypeIdentifier reference
      */
     RTPS_DllAPI const fastrtps::types::TypeIdentifier& get() const
@@ -2332,6 +2412,7 @@ public:
 
     /**
      * @brief Copy constructor
+     *
      * @param type Another instance of TypeObjectV1
      */
     RTPS_DllAPI TypeObjectV1(
@@ -2344,6 +2425,7 @@ public:
 
     /**
      * @brief Constructor using a TypeObject
+     *
      * @param type TypeObject to be set
      */
     RTPS_DllAPI TypeObjectV1(
@@ -2356,6 +2438,7 @@ public:
 
     /**
      * @brief Move constructor
+     *
      * @param type Another instance of TypeObjectV1
      */
     RTPS_DllAPI TypeObjectV1(
@@ -2409,6 +2492,7 @@ public:
 
     /**
      * @brief Getter for the TypeObject
+     *
      * @return TypeObject reference
      */
     RTPS_DllAPI const fastrtps::types::TypeObject& get() const
@@ -2443,6 +2527,7 @@ public:
 
     /**
      * @brief Copy constructor
+     *
      * @param type Another instance of TypeInformation
      */
     RTPS_DllAPI TypeInformation(
@@ -2456,6 +2541,7 @@ public:
 
     /**
      * @brief Constructor using a fastrtps::types::TypeInformation
+     *
      * @param info fastrtps::types::TypeInformation to be set
      */
     RTPS_DllAPI TypeInformation(
@@ -2469,6 +2555,7 @@ public:
 
     /**
      * @brief Move Constructor
+     *
      * @param type Another instance of TypeInformation
      */
     RTPS_DllAPI TypeInformation(
@@ -2518,6 +2605,7 @@ public:
 
     /**
      * @brief Check if it is assigned
+     *
      * @return true if assigned, false if not
      */
     RTPS_DllAPI bool assigned() const
@@ -2527,6 +2615,7 @@ public:
 
     /**
      * @brief Setter for assigned boolean
+     *
      * @param value Boolean to be set
      */
     RTPS_DllAPI void assigned(
@@ -2587,6 +2676,8 @@ public:
                (this->throughput_controller == b.throughput_controller) &&
                (this->default_unicast_locator_list == b.default_unicast_locator_list) &&
                (this->default_multicast_locator_list == b.default_multicast_locator_list) &&
+               (this->default_external_unicast_locators == b.default_external_unicast_locators) &&
+               (this->ignore_non_matching_locators == b.ignore_non_matching_locators) &&
                QosPolicy::operator ==(b);
     }
 
@@ -2599,16 +2690,16 @@ public:
     //! Optionally allows user to define the GuidPrefix_t
     fastrtps::rtps::GuidPrefix_t prefix;
 
-    //!Participant ID <br> By default, -1.
+    //! Participant ID <br> By default, -1.
     int32_t participant_id;
 
     //! Builtin parameters.
     fastrtps::rtps::BuiltinAttributes builtin;
 
-    //!Port Parameters
+    //! Port Parameters
     fastrtps::rtps::PortParameters port;
 
-    //!Throughput controller parameters. Leave default for uncontrolled flow.
+    //! Throughput controller parameters. Leave default for uncontrolled flow.
     fastrtps::rtps::ThroughputControllerDescriptor throughput_controller;
 
     /**
@@ -2619,9 +2710,19 @@ public:
 
     /**
      * Default list of Multicast Locators to be used for any Endpoint defined inside this RTPSParticipant in the
-     * case that it was defined with NO UnicastLocators. This is usually left empty.
+     * case that it was defined with NO MulticastLocators. This is usually left empty.
      */
     rtps::LocatorList default_multicast_locator_list;
+
+    /**
+     * The collection of external locators to use for communication on user created topics.
+     */
+    rtps::ExternalLocators default_external_unicast_locators;
+
+    /**
+     * Whether locators that don't match with the announced locators should be kept.
+     */
+    bool ignore_non_matching_locators = false;
 };
 
 //! Qos Policy to configure the transport layer
@@ -2679,17 +2780,12 @@ public:
     uint32_t listen_socket_buffer_size;
 };
 
-//!Qos Policy to configure the endpoint
+//! Qos Policy to configure the endpoint
 class RTPSEndpointQos
 {
 public:
 
-    RTPS_DllAPI RTPSEndpointQos()
-        : user_defined_id(-1)
-        , entity_id(-1)
-        , history_memory_policy(fastrtps::rtps::PREALLOCATED_MEMORY_MODE)
-    {
-    }
+    RTPS_DllAPI RTPSEndpointQos() = default;
 
     virtual RTPS_DllAPI ~RTPSEndpointQos() = default;
 
@@ -2699,28 +2795,36 @@ public:
         return (this->unicast_locator_list == b.unicast_locator_list) &&
                (this->multicast_locator_list == b.multicast_locator_list) &&
                (this->remote_locator_list == b.remote_locator_list) &&
+               (this->external_unicast_locators == b.external_unicast_locators) &&
+               (this->ignore_non_matching_locators == b.ignore_non_matching_locators) &&
                (this->user_defined_id == b.user_defined_id) &&
                (this->entity_id == b.entity_id) &&
                (this->history_memory_policy == b.history_memory_policy);
     }
 
-    //!Unicast locator list
+    //! Unicast locator list
     rtps::LocatorList unicast_locator_list;
 
-    //!Multicast locator list
+    //! Multicast locator list
     rtps::LocatorList multicast_locator_list;
 
-    //!Remote locator list
+    //! Remote locator list
     rtps::LocatorList remote_locator_list;
 
-    //!User Defined ID, used for StaticEndpointDiscovery. <br> By default, -1.
-    int16_t user_defined_id;
+    //! The collection of external locators to use for communication.
+    fastdds::rtps::ExternalLocators external_unicast_locators;
 
-    //!Entity ID, if the user wants to specify the EntityID of the endpoint. <br> By default, -1.
-    int16_t entity_id;
+    //! Whether locators that don't match with the announced locators should be kept.
+    bool ignore_non_matching_locators = false;
 
-    //!Underlying History memory policy. <br> By default, PREALLOCATED_MEMORY_MODE.
-    fastrtps::rtps::MemoryManagementPolicy_t history_memory_policy;
+    //! User Defined ID, used for StaticEndpointDiscovery. <br> By default, -1.
+    int16_t user_defined_id = -1;
+
+    //! Entity ID, if the user wants to specify the EntityID of the endpoint. <br> By default, -1.
+    int16_t entity_id = -1;
+
+    //! Underlying History memory policy. <br> By default, PREALLOCATED_MEMORY_MODE.
+    fastrtps::rtps::MemoryManagementPolicy_t history_memory_policy = fastrtps::rtps::PREALLOCATED_MEMORY_MODE;
 };
 
 //!Qos Policy to configure the limit of the writer resources
@@ -2732,6 +2836,8 @@ public:
      * @brief Constructor
      */
     RTPS_DllAPI WriterResourceLimitsQos()
+        : matched_subscriber_allocation()
+        , reader_filters_allocation(0, 32u, 1u)
     {
     }
 
@@ -2743,11 +2849,14 @@ public:
     bool operator ==(
             const WriterResourceLimitsQos& b) const
     {
-        return (this->matched_subscriber_allocation == b.matched_subscriber_allocation);
+        return (matched_subscriber_allocation == b.matched_subscriber_allocation) &&
+               (reader_filters_allocation == b.reader_filters_allocation);
     }
 
     //!Matched subscribers allocation limits.
     fastrtps::ResourceLimitedContainerConfig matched_subscriber_allocation;
+    //!Reader filters allocation limits.
+    fastrtps::ResourceLimitedContainerConfig reader_filters_allocation;
 };
 
 /**
@@ -2774,6 +2883,7 @@ enum DataSharingKind : fastrtps::rtps::octet
 
 /**
  * Qos Policy to configure the data sharing
+ *
  * @note Immutable Qos Policy
  */
 class DataSharingQosPolicy : public Parameter_t, public QosPolicy
@@ -2798,6 +2908,7 @@ public:
 
     /**
      * @brief Copy constructor
+     *
      * @param b Another DataSharingQosPolicy instance
      */
     RTPS_DllAPI DataSharingQosPolicy(
