@@ -19,17 +19,18 @@
 #ifndef _FASTDDS_RTPSPARTICIPANTPARAMETERS_H_
 #define _FASTDDS_RTPSPARTICIPANTPARAMETERS_H_
 
-#include <fastdds/rtps/common/Time_t.h>
-#include <fastdds/rtps/common/Locator.h>
-#include <fastdds/rtps/common/PortParameters.h>
+#include <fastdds/rtps/attributes/ExternalLocators.hpp>
 #include <fastdds/rtps/attributes/PropertyPolicy.h>
-#include <fastdds/rtps/flowcontrol/ThroughputControllerDescriptor.h>
-#include <fastdds/rtps/transport/TransportInterface.h>
-#include <fastdds/rtps/resources/ResourceManagement.h>
-#include <fastrtps/utils/fixed_size_string.hpp>
 #include <fastdds/rtps/attributes/RTPSParticipantAllocationAttributes.hpp>
 #include <fastdds/rtps/attributes/ServerAttributes.h>
+#include <fastdds/rtps/common/Locator.h>
+#include <fastdds/rtps/common/PortParameters.h>
+#include <fastdds/rtps/common/Time_t.h>
 #include <fastdds/rtps/flowcontrol/FlowControllerDescriptor.hpp>
+#include <fastdds/rtps/flowcontrol/ThroughputControllerDescriptor.h>
+#include <fastdds/rtps/resources/ResourceManagement.h>
+#include <fastdds/rtps/transport/TransportInterface.h>
+#include <fastrtps/utils/fixed_size_string.hpp>
 
 #include <memory>
 #include <sstream>
@@ -374,18 +375,21 @@ public:
     //! Discovery protocol related attributes
     DiscoverySettings discovery_config;
 
-    //!Indicates to use the WriterLiveliness protocol.
+    //! Indicates to use the WriterLiveliness protocol.
     bool use_WriterLivelinessProtocol = true;
 
-    //!TypeLookup Service settings
+    //! TypeLookup Service settings
     TypeLookupSettings typelookup_config;
 
-    //!Metatraffic Unicast Locator List
+    //! Metatraffic Unicast Locator List
     LocatorList_t metatrafficUnicastLocatorList;
 
-    //!Metatraffic Multicast Locator List.
+    //! Metatraffic Multicast Locator List.
     LocatorList_t metatrafficMulticastLocatorList;
 
+    //! The collection of external locators to use for communication on metatraffic topics.
+    fastdds::rtps::ExternalLocators metatraffic_external_unicast_locators;
+
     //! Initial peers.
     LocatorList_t initialPeersList;
 
@@ -406,7 +410,7 @@ public:
     //! Mutation tries if the port is being used.
     uint32_t mutation_tries = 100u;
 
-    //!Set to true to avoid multicast traffic on builtin endpoints
+    //! Set to true to avoid multicast traffic on builtin endpoints
     bool avoid_builtin_multicast = true;
 
     BuiltinAttributes() = default;
@@ -422,6 +426,7 @@ public:
                (typelookup_config.use_server == b.typelookup_config.use_server) &&
                (this->metatrafficUnicastLocatorList == b.metatrafficUnicastLocatorList) &&
                (this->metatrafficMulticastLocatorList == b.metatrafficMulticastLocatorList) &&
+               (this->metatraffic_external_unicast_locators == b.metatraffic_external_unicast_locators) &&
                (this->initialPeersList == b.initialPeersList) &&
                (this->readerHistoryMemoryPolicy == b.readerHistoryMemoryPolicy) &&
                (this->readerPayloadSize == b.readerPayloadSize) &&
@@ -462,6 +467,8 @@ public:
         return (this->name == b.name) &&
                (this->defaultUnicastLocatorList == b.defaultUnicastLocatorList) &&
                (this->defaultMulticastLocatorList == b.defaultMulticastLocatorList) &&
+               (this->default_external_unicast_locators == b.default_external_unicast_locators) &&
+               (this->ignore_non_matching_locators == b.ignore_non_matching_locators) &&
                (this->sendSocketBufferSize == b.sendSocketBufferSize) &&
                (this->listenSocketBufferSize == b.listenSocketBufferSize) &&
                (this->builtin == b.builtin) &&
@@ -483,10 +490,20 @@ public:
 
     /**
      * Default list of Multicast Locators to be used for any Endpoint defined inside this RTPSParticipant in the
-     * case that it was defined with NO UnicastLocators. This is usually left empty.
+     * case that it was defined with NO MulticastLocators. This is usually left empty.
      */
     LocatorList_t defaultMulticastLocatorList;
 
+    /**
+     * The collection of external locators to use for communication on user created topics.
+     */
+    fastdds::rtps::ExternalLocators default_external_unicast_locators;
+
+    /**
+     * Whether locators that don't match with the announced locators should be kept.
+     */
+    bool ignore_non_matching_locators = false;
+
     /*!
      * @brief Send socket buffer size for the send resource. Zero value indicates to use default system buffer size.
      * Default value: 0.
@@ -510,37 +527,38 @@ public:
     //! Builtin parameters.
     BuiltinAttributes builtin;
 
-    //!Port Parameters
+    //! Port Parameters
     PortParameters port;
 
-    //!User Data of the participant
+    //! User Data of the participant
     std::vector<octet> userData;
 
-    //!Participant ID
+    //! Participant ID
     int32_t participantID;
 
-    //!Throughput controller parameters. Leave default for uncontrolled flow.
+    //! Throughput controller parameters. Leave default for uncontrolled flow.
     ThroughputControllerDescriptor throughputController;
 
-    //!User defined transports to use alongside or in place of builtins.
+    //! User defined transports to use alongside or in place of builtins.
     std::vector<std::shared_ptr<fastdds::rtps::TransportDescriptorInterface>> userTransports;
 
-    //!Set as false to disable the default UDPv4 implementation.
+    //! Set as false to disable the default UDPv4 implementation.
     bool useBuiltinTransports;
-    //!Holds allocation limits affecting collections managed by a participant.
+
+    //! Holds allocation limits affecting collections managed by a participant.
     RTPSParticipantAllocationAttributes allocation;
 
     //! Property policies
     PropertyPolicy properties;
 
-    //!Set the name of the participant.
+    //! Set the name of the participant.
     inline void setName(
             const char* nam)
     {
         name = nam;
     }
 
-    //!Get the name of the participant.
+    //! Get the name of the participant.
     inline const char* getName() const
     {
         return name.c_str();
@@ -551,7 +569,7 @@ public:
 
 private:
 
-    //!Name of the participant.
+    //! Name of the participant.
     string_255 name;
 };
 
