@@ -17,12 +17,18 @@
  *
  */
 
+#include <chrono>
+#include <string>
+
+#include <asio.hpp>
+
 #include <fastdds/domain/DomainParticipantImpl.hpp>
 
+#include <fastdds/core/policy/QosPolicyUtils.hpp>
 #include <fastdds/dds/builtin/typelookup/TypeLookupManager.hpp>
 #include <fastdds/dds/domain/DomainParticipant.hpp>
-#include <fastdds/dds/domain/DomainParticipantListener.hpp>
 #include <fastdds/dds/domain/DomainParticipantFactory.hpp>
+#include <fastdds/dds/domain/DomainParticipantListener.hpp>
 #include <fastdds/dds/log/Log.hpp>
 #include <fastdds/dds/publisher/DataWriter.hpp>
 #include <fastdds/dds/publisher/Publisher.hpp>
@@ -30,34 +36,32 @@
 #include <fastdds/dds/subscriber/Subscriber.hpp>
 #include <fastdds/dds/topic/IContentFilterFactory.hpp>
 #include <fastdds/dds/topic/TypeSupport.hpp>
-
-#include <fastdds/rtps/RTPSDomain.h>
+#include <fastdds/publisher/PublisherImpl.hpp>
+#include <fastdds/rtps/attributes/PropertyPolicy.h>
 #include <fastdds/rtps/attributes/RTPSParticipantAttributes.h>
 #include <fastdds/rtps/builtin/liveliness/WLP.h>
 #include <fastdds/rtps/participant/ParticipantDiscoveryInfo.h>
 #include <fastdds/rtps/participant/RTPSParticipant.h>
 #include <fastdds/rtps/reader/ReaderDiscoveryInfo.h>
+#include <fastdds/rtps/RTPSDomain.h>
 #include <fastdds/rtps/writer/WriterDiscoveryInfo.h>
-
 #include <fastrtps/attributes/PublisherAttributes.h>
 #include <fastrtps/attributes/SubscriberAttributes.h>
-
-#include <fastrtps/types/DynamicTypeBuilderFactory.h>
 #include <fastrtps/types/DynamicPubSubType.h>
 #include <fastrtps/types/DynamicType.h>
+#include <fastrtps/types/DynamicTypeBuilderFactory.h>
 #include <fastrtps/types/DynamicTypeMember.h>
 #include <fastrtps/types/TypeObjectFactory.h>
-
 #include <fastrtps/xmlparser/XMLProfileManager.h>
 
-#include <fastdds/publisher/PublisherImpl.hpp>
+#include <fastdds/publisher/DataWriterImpl.hpp>
 #include <fastdds/subscriber/SubscriberImpl.hpp>
-#include <fastdds/topic/TopicImpl.hpp>
 #include <fastdds/topic/ContentFilteredTopicImpl.hpp>
-
+#include <fastdds/topic/TopicImpl.hpp>
+#include <fastdds/topic/TopicProxy.hpp>
+#include <fastdds/topic/TopicProxyFactory.hpp>
 #include <rtps/RTPSDomainImpl.hpp>
-
-#include <chrono>
+#include <utils/SystemInfo.hpp>
 
 namespace eprosima {
 namespace fastdds {
@@ -166,6 +170,32 @@ DomainParticipantImpl::DomainParticipantImpl(
     // Pre calculate participant id and generated guid
     participant_id_ = qos_.wire_protocol().participant_id;
     eprosima::fastrtps::rtps::RTPSDomainImpl::create_participant_guid(participant_id_, guid_);
+
+    /* Fill physical data properties if they are found and empty */
+    std::string* property_value = fastrtps::rtps::PropertyPolicyHelper::find_property(
+        qos_.properties(), parameter_policy_physical_data_host);
+    if (nullptr != property_value && property_value->empty())
+    {
+        property_value->assign(asio::ip::host_name() + ":" + std::to_string(utils::default_domain_id()));
+    }
+
+    property_value = fastrtps::rtps::PropertyPolicyHelper::find_property(
+        qos_.properties(), parameter_policy_physical_data_user);
+    if (nullptr != property_value && property_value->empty())
+    {
+        std::string username = "unknown";
+        if (ReturnCode_t::RETCODE_OK == SystemInfo::get_username(username))
+        {
+            property_value->assign(username);
+        }
+    }
+
+    property_value = fastrtps::rtps::PropertyPolicyHelper::find_property(
+        qos_.properties(), parameter_policy_physical_data_process);
+    if (nullptr != property_value && property_value->empty())
+    {
+        property_value->assign(std::to_string(SystemInfo::instance().process_id()));
+    }
 }
 
 void DomainParticipantImpl::disable()
@@ -284,7 +314,6 @@ ReturnCode_t DomainParticipantImpl::enable()
 
     guid_ = part->getGuid();
     rtps_participant_ = part;
-    rtps_participant_->enable();
 
     rtps_participant_->set_check_type_function(
         [this](const std::string& type_name) -> bool
@@ -300,7 +329,7 @@ ReturnCode_t DomainParticipantImpl::enable()
 
             for (auto topic : topics_)
             {
-                topic.second->user_topic_->enable();
+                topic.second->enable_topic();
             }
         }
 
@@ -326,6 +355,8 @@ ReturnCode_t DomainParticipantImpl::enable()
         }
     }
 
+    rtps_participant_->enable();
+
     return ReturnCode_t::RETCODE_OK;
 }
 
@@ -438,6 +469,53 @@ ReturnCode_t DomainParticipantImpl::delete_subscriber(
     return ReturnCode_t::RETCODE_ERROR;
 }
 
+Topic* DomainParticipantImpl::find_topic(
+        const std::string& topic_name,
+        const fastrtps::Duration_t& timeout)
+{
+    auto find_fn = [this, &topic_name]()
+            {
+                return topics_.count(topic_name) > 0;
+            };
+
+    std::unique_lock<std::mutex> lock(mtx_topics_);
+    if (timeout.is_infinite())
+    {
+        cond_topics_.wait(lock, find_fn);
+    }
+    else
+    {
+        auto duration = std::chrono::seconds(timeout.seconds) + std::chrono::nanoseconds(timeout.nanosec);
+        if (!cond_topics_.wait_for(lock, duration, find_fn))
+        {
+            return nullptr;
+        }
+    }
+
+    Topic* ret_val = topics_[topic_name]->create_topic()->get_topic();
+
+    InstanceHandle_t topic_handle;
+    create_instance_handle(topic_handle);
+    ret_val->set_instance_handle(topic_handle);
+    topics_by_handle_[topic_handle] = ret_val;
+
+    return ret_val;
+}
+
+void DomainParticipantImpl::set_topic_listener(
+        const TopicProxyFactory* factory,
+        TopicImpl* impl,
+        TopicListener* listener,
+        const StatusMask& mask)
+{
+    std::lock_guard<std::mutex> lock(mtx_topics_);
+    impl->set_listener(listener);
+    factory->for_each([mask](const std::unique_ptr<TopicProxy>& proxy)
+            {
+                proxy->get_topic()->status_mask_ = mask;
+            });
+}
+
 ReturnCode_t DomainParticipantImpl::delete_topic(
         const Topic* topic)
 {
@@ -446,30 +524,36 @@ ReturnCode_t DomainParticipantImpl::delete_topic(
         return ReturnCode_t::RETCODE_BAD_PARAMETER;
     }
 
-    if (participant_ != topic->get_participant())
-    {
-        return ReturnCode_t::RETCODE_PRECONDITION_NOT_MET;
-    }
-
     std::lock_guard<std::mutex> lock(mtx_topics_);
-    auto it = topics_.find(topic->get_name());
-
-    if (it != topics_.end())
+    auto handle_it = std::find_if(topics_by_handle_.begin(), topics_by_handle_.end(),
+                    [topic](const decltype(topics_by_handle_)::value_type& item)
+                    {
+                        return item.second == topic;
+                    });
+    if (handle_it != topics_by_handle_.end())
     {
-        assert(topic->get_instance_handle() == it->second->get_topic()->get_instance_handle()
-                && "The topic instance handle does not match the topic implementation instance handle");
-        if (it->second->is_referenced())
+        auto it = topics_.find(topic->get_name());
+        assert(it != topics_.end() && "Topic found by handle but factory not found");
+        InstanceHandle_t handle = topic->get_instance_handle();
+
+        TopicProxy* proxy = dynamic_cast<TopicProxy*>(topic->get_impl());
+        assert(nullptr != proxy);
+        auto ret_code = it->second->delete_topic(proxy);
+        if (ReturnCode_t::RETCODE_OK == ret_code)
         {
-            return ReturnCode_t::RETCODE_PRECONDITION_NOT_MET;
+            topics_by_handle_.erase(handle);
+
+            if (it->second->can_be_deleted())
+            {
+                auto factory = it->second;
+                topics_.erase(it);
+                delete factory;
+            }
         }
-        it->second->set_listener(nullptr);
-        topics_by_handle_.erase(topic->get_instance_handle());
-        delete it->second;
-        topics_.erase(it);
-        return ReturnCode_t::RETCODE_OK;
+        return ret_code;
     }
 
-    return ReturnCode_t::RETCODE_ERROR;
+    return ReturnCode_t::RETCODE_PRECONDITION_NOT_MET;
 }
 
 ContentFilteredTopic* DomainParticipantImpl::create_contentfilteredtopic(
@@ -508,7 +592,7 @@ ContentFilteredTopic* DomainParticipantImpl::create_contentfilteredtopic(
         return nullptr;
     }
 
-    TopicImpl* topic_impl = dynamic_cast<TopicImpl*>(related_topic->get_impl());
+    TopicProxy* topic_impl = dynamic_cast<TopicProxy*>(related_topic->get_impl());
     assert(nullptr != topic_impl);
     const TypeSupport& type = topic_impl->get_type();
     LoanableSequence<const char*>::size_type n_params;
@@ -535,12 +619,13 @@ ContentFilteredTopic* DomainParticipantImpl::create_contentfilteredtopic(
     ContentFilteredTopic* topic;
     topic = new ContentFilteredTopic(name, related_topic, filter_expression, expression_parameters);
     ContentFilteredTopicImpl* content_topic_impl = static_cast<ContentFilteredTopicImpl*>(topic->get_impl());
-    content_topic_impl->filter_class_name = filter_class_name;
+    content_topic_impl->filter_property.filter_class_name = filter_class_name;
     content_topic_impl->filter_factory = filter_factory;
     content_topic_impl->filter_instance = filter_instance;
+    content_topic_impl->update_signature();
 
     // Save the topic into the map
-    filtered_topics_.emplace(std::make_pair(name, topic));
+    filtered_topics_.emplace(std::make_pair(name, std::unique_ptr<ContentFilteredTopic>(topic)));
 
     return topic;
 }
@@ -623,19 +708,30 @@ ReturnCode_t DomainParticipantImpl::unregister_content_filter_factory(
 
     for (auto& topic : filtered_topics_)
     {
-        if (topic.second->impl_->filter_class_name == filter_class_name)
+        if (topic.second->impl_->filter_property.filter_class_name == filter_class_name)
         {
             return ReturnCode_t::RETCODE_PRECONDITION_NOT_MET;
         }
     }
 
+    for (auto& pub : publishers_)
+    {
+        for (auto& topic : pub.second->writers_)
+        {
+            for (auto& wr : topic.second)
+            {
+                wr->filter_is_being_removed(filter_class_name);
+            }
+        }
+    }
+
     filter_factories_.erase(it);
 
     return ReturnCode_t::RETCODE_OK;
 }
 
 IContentFilterFactory* DomainParticipantImpl::find_content_filter_factory(
-        const char* filter_class_name) const
+        const char* filter_class_name)
 {
     auto it = filter_factories_.find(filter_class_name);
     if (it != filter_factories_.end())
@@ -648,8 +744,7 @@ IContentFilterFactory* DomainParticipantImpl::find_content_filter_factory(
         return nullptr;
     }
 
-    // TODO(Miguel C): Create SQLFilter::DDSFilterFactory
-    return nullptr;
+    return &dds_sql_filter_factory_;
 }
 
 const InstanceHandle_t& DomainParticipantImpl::get_instance_handle() const
@@ -863,12 +958,12 @@ ReturnCode_t DomainParticipantImpl::delete_contained_entities()
 
     std::lock_guard<std::mutex> lock_topics(mtx_topics_);
 
-    auto it_topics = topics_.begin();
+    filtered_topics_.clear();
+    topics_by_handle_.clear();
 
+    auto it_topics = topics_.begin();
     while (it_topics != topics_.end())
     {
-        it_topics->second->set_listener(nullptr);
-        topics_by_handle_.erase(it_topics->second->get_topic()->get_instance_handle());
         delete it_topics->second;
         it_topics = topics_.erase(it_topics);
     }
@@ -1259,15 +1354,14 @@ Topic* DomainParticipantImpl::create_topic(
     InstanceHandle_t topic_handle;
     create_instance_handle(topic_handle);
 
-    //TODO CONSTRUIR LA IMPLEMENTACION DENTRO DEL OBJETO DEL USUARIO.
-    TopicImpl* topic_impl = new TopicImpl(this, type_support, qos, listener);
-    Topic* topic = new Topic(topic_name, type_name, topic_impl, mask);
-    topic_impl->user_topic_ = topic;
+    TopicProxyFactory* factory = new TopicProxyFactory(this, topic_name, mask, type_support, qos, listener);
+    TopicProxy* proxy = factory->create_topic();
+    Topic* topic = proxy->get_topic();
     topic->set_instance_handle(topic_handle);
 
     //SAVE THE TOPIC INTO MAPS
     topics_by_handle_[topic_handle] = topic;
-    topics_[topic_name] = topic_impl;
+    topics_[topic_name] = factory;
 
     // Enable topic if appropriate
     if (enabled && qos_.entity_factory().autoenable_created_entities)
@@ -1277,6 +1371,8 @@ Topic* DomainParticipantImpl::create_topic(
         (void)ret_topic_enable;
     }
 
+    cond_topics_.notify_all();
+
     return topic;
 }
 
@@ -1307,7 +1403,7 @@ TopicDescription* DomainParticipantImpl::lookup_topicdescription(
     auto it = topics_.find(topic_name);
     if (it != topics_.end())
     {
-        return it->second->user_topic_;
+        return it->second->get_topic()->get_topic();
     }
 
     auto filtered_it = filtered_topics_.find(topic_name);
