@@ -19,6 +19,8 @@
 #ifndef _FASTRTPS_DATAWRITERIMPL_HPP_
 #define _FASTRTPS_DATAWRITERIMPL_HPP_
 
+#include <memory>
+
 #include <fastdds/dds/core/status/BaseStatus.hpp>
 #include <fastdds/dds/core/status/IncompatibleQosStatus.hpp>
 #include <fastdds/dds/publisher/DataWriter.hpp>
@@ -31,15 +33,19 @@
 #include <fastdds/rtps/common/LocatorList.hpp>
 #include <fastdds/rtps/common/Guid.h>
 #include <fastdds/rtps/common/WriteParams.h>
+#include <fastdds/rtps/history/IChangePool.h>
 #include <fastdds/rtps/history/IPayloadPool.h>
+#include <fastdds/rtps/interfaces/IReaderDataFilter.hpp>
 #include <fastdds/rtps/writer/WriterListener.h>
 
-#include <fastrtps/publisher/PublisherHistory.h>
 #include <fastrtps/qos/DeadlineMissedStatus.h>
 #include <fastrtps/qos/LivelinessLostStatus.h>
 
 #include <fastrtps/types/TypesBase.h>
 
+#include <fastdds/publisher/DataWriterHistory.hpp>
+#include <fastdds/publisher/filtering/ReaderFilterCollection.hpp>
+
 #include <rtps/common/PayloadInfo_t.hpp>
 #include <rtps/history/ITopicPayloadPool.h>
 #include <rtps/DataSharing/DataSharingPayloadPool.hpp>
@@ -78,7 +84,7 @@ class Publisher;
  * Class DataWriterImpl, contains the actual implementation of the behaviour of the DataWriter.
  * @ingroup FASTDDS_MODULE
  */
-class DataWriterImpl
+class DataWriterImpl : protected rtps::IReaderDataFilter
 {
     using LoanInitializationKind = DataWriter::LoanInitializationKind;
     using PayloadInfo_t = eprosima::fastrtps::rtps::detail::PayloadInfo_t;
@@ -116,8 +122,23 @@ public:
 
     virtual ~DataWriterImpl();
 
+    /**
+     * Enable this object.
+     * The required lower layer entities will be created.
+     *
+     * @pre This method has not previously returned ReturnCode_t::RETCODE_OK
+     *
+     * @return ReturnCode_t::RETCODE_OK if all the lower layer entities have been correctly created.
+     * @return Other standard return codes on error.
+     */
     virtual ReturnCode_t enable();
 
+    /**
+     * Check if the preconditions to delete this object are met.
+     *
+     * @return ReturnCode_t::RETCODE_PRECONDITION_NOT_MET if the preconditions to delete this object are not met.
+     * @return ReturnCode_t::RETCODE_OK if it is safe to delete this object.
+     */
     ReturnCode_t check_delete_preconditions();
 
     /**
@@ -148,42 +169,60 @@ public:
 
     /**
      * Write data to the topic.
-     * @param data Pointer to the data
-     * @return True if correct
-     * @par Calling example:
-     * @snippet fastrtps_example.cpp ex_PublisherWrite
+     *
+     * @param data Pointer to the data.
+     *
+     * @return true if data is correctly delivered to the lower layers, false otherwise.
      */
     bool write(
             void* data);
 
     /**
      * Write data with params to the topic.
-     * @param data Pointer to the data
+     *
+     * @param data Pointer to the data.
      * @param params Extra write parameters.
-     * @return True if correct
-     * @par Calling example:
-     * @snippet fastrtps_example.cpp ex_PublisherWrite
+     *
+     * @return true if data is correctly delivered to the lower layers, false otherwise.
      */
     bool write(
             void* data,
             fastrtps::rtps::WriteParams& params);
 
     /**
-     * Write data with handle.
-     * @param data Pointer to the data
-     * @param handle InstanceHandle_t.
-     * @return True if correct
-     * @par Calling example:
-     * @snippet fastrtps_example.cpp ex_PublisherWrite
+     * @brief Implementation of the DDS `write` operation.
+     *
+     * @param[in] data    Pointer to the data to publish.
+     * @param[in] handle  Handle of the instance to update. The special value @c HANDLE_NIL can be used to indicate
+     *                    that the instance should be automatically calculated.
+     *
+     * @return any of the standard return codes.
      */
     ReturnCode_t write(
             void* data,
             const InstanceHandle_t& handle);
 
-    /*!
+    /**
+     * @brief Implementation of the DDS `write_w_timestamp` operation.
+     *
+     * @param[in] data        Pointer to the data to publish.
+     * @param[in] handle      Handle of the instance to update. The special value @c HANDLE_NIL can be used to indicate
+     *                        that the instance should be automatically calculated.
+     * @param[in] timestamp   Timestamp to associate to the sample info of the published data.
+     *
+     * @return any of the standard return codes.
+     */
+    ReturnCode_t write_w_timestamp(
+            void* data,
+            const InstanceHandle_t& handle,
+            const fastrtps::Time_t& timestamp);
+
+    /**
      * @brief Implementation of the DDS `register_instance` operation.
-     * It deduces the instance's key and tries to get resources in the PublisherHistory.
+     * It deduces the instance's key and tries to get resources in the DataWriterHistory.
+     *
      * @param[in] instance Sample used to get the instance's key.
+     *
      * @return Handle containing the instance's key.
      * This handle could be used in successive `write` or `dispose` operations.
      * In case of error, HANDLE_NIL will be returned.
@@ -191,16 +230,33 @@ public:
     InstanceHandle_t register_instance(
             void* instance);
 
-    /*!
+    /**
+     * @brief Implementation of the DDS `register_instance_w_timestamp` operation.
+     * It deduces the instance's key and tries to get resources in the DataWriterHistory.
+     *
+     * @param[in] instance Sample used to get the instance's key.
+     * @param[in] timestamp Timestamp to set on the instance registration operation.
+     *
+     * @return Handle containing the instance's key.
+     * This handle could be used in successive `write` or `dispose` operations.
+     * In case of error, HANDLE_NIL will be returned.
+     */
+    InstanceHandle_t register_instance_w_timestamp(
+            void* instance,
+            const fastrtps::Time_t& timestamp);
+
+    /**
      * @brief Implementation of the DDS `unregister_instance` and `dispose` operations.
      * It sends a CacheChange_t with a kind that depends on the `dispose` parameter and
      * `writer_data_lifecycle` QoS.
-     * @param[in] instance Sample used to deduce instance's key in case of `handle` parameter is HANDLE_NIL.
-     * @param[in] handle Instance's key to be unregistered or disposed.
-     * @param[in] dispose If it is `false`, a CacheChange_t with kind set to NOT_ALIVE_UNREGISTERED is sent, or if
-     * `writer_data_lifecycle.autodispose_unregistered_instances` is `true` then it is sent with kind set to
-     * NOT_ALIVE_DISPOSED_UNREGISTERED.
-     * If `dispose` is `true`, a CacheChange_t with kind set to NOT_ALIVE_DISPOSED is sent.
+     *
+     * @param[in] instance  Sample used to deduce instance's key in case of `handle` parameter is HANDLE_NIL.
+     * @param[in] handle    Instance's key to be unregistered or disposed.
+     * @param[in] dispose   If `dispose` is `false`, a CacheChange_t with kind set to NOT_ALIVE_UNREGISTERED is sent,
+     *                      or if `writer_data_lifecycle.autodispose_unregistered_instances` is `true` then it is sent
+     *                      with kind set to NOT_ALIVE_DISPOSED_UNREGISTERED.
+     *                      If `dispose` is `true`, a CacheChange_t with kind set to NOT_ALIVE_DISPOSED is sent.
+     *
      * @return Returns the operation's result.
      * If the operation finishes successfully, ReturnCode_t::RETCODE_OK is returned.
      */
@@ -209,6 +265,28 @@ public:
             const InstanceHandle_t& handle,
             bool dispose = false);
 
+    /**
+     * @brief Implementation of the DDS `unregister_instance_w_timestamp` and `dispose_w_timestamp` operations.
+     * It sends a CacheChange_t with a kind that depends on the `dispose` parameter and
+     * `writer_data_lifecycle` QoS.
+     *
+     * @param[in] instance  Sample used to deduce instance's key in case of `handle` parameter is HANDLE_NIL.
+     * @param[in] handle    Instance's key to be unregistered or disposed.
+     * @param[in] timestamp Source timestamp to set on the CacheChange_t being sent.
+     * @param[in] dispose   If `dispose` is `false`, a CacheChange_t with kind set to NOT_ALIVE_UNREGISTERED is sent,
+     *                      or if `writer_data_lifecycle.autodispose_unregistered_instances` is `true` then it is sent
+     *                      with kind set to NOT_ALIVE_DISPOSED_UNREGISTERED.
+     *                      If `dispose` is `true`, a CacheChange_t with kind set to NOT_ALIVE_DISPOSED is sent.
+     *
+     * @return Returns the operation's result.
+     * If the operation finishes successfully, ReturnCode_t::RETCODE_OK is returned.
+     */
+    ReturnCode_t unregister_instance_w_timestamp(
+            void* instance,
+            const InstanceHandle_t& handle,
+            const fastrtps::Time_t& timestamp,
+            bool dispose = false);
+
     /**
      *
      * @return
@@ -255,11 +333,23 @@ public:
     ReturnCode_t set_listener(
             DataWriterListener* listener);
 
-    /* TODO
-       bool get_key_value(
+    /**
+     * This operation can be used to retrieve the instance key that corresponds to an
+     * @ref eprosima::fastdds::dds::Entity::instance_handle_ "instance_handle".
+     * The operation will only fill the fields that form the key inside the key_holder instance.
+     *
+     * This operation may return BAD_PARAMETER if the InstanceHandle_t handle does not correspond to an existing
+     * data-object known to the DataWriter. If the implementation is not able to check invalid handles then the result
+     * in this situation is unspecified.
+     *
+     * @param[in,out] key_holder  Sample where the key fields will be returned.
+     * @param[in] handle          Handle to the instance to retrieve the key values from.
+     *
+     * @return Any of the standard return codes.
+     */
+    ReturnCode_t get_key_value(
             void* key_holder,
             const InstanceHandle_t& handle);
-     */
 
     ReturnCode_t get_liveliness_lost_status(
             LivelinessLostStatus& status);
@@ -290,8 +380,17 @@ public:
     ReturnCode_t get_sending_locators(
             rtps::LocatorList& locators) const;
 
+    /**
+     * Called from the DomainParticipant when a filter factory is being unregistered.
+     *
+     * @param filter_class_name  The class name under which the factory was registered.
+     */
+    void filter_is_being_removed(
+            const char* filter_class_name);
+
 protected:
 
+    using IChangePool = eprosima::fastrtps::rtps::IChangePool;
     using IPayloadPool = eprosima::fastrtps::rtps::IPayloadPool;
     using ITopicPayloadPool = eprosima::fastrtps::rtps::ITopicPayloadPool;
 
@@ -307,8 +406,8 @@ protected:
 
     DataWriterQos qos_;
 
-    //!Publisher History
-    fastrtps::PublisherHistory history_;
+    //!History
+    DataWriterHistory history_;
 
     //! DataWriterListener
     DataWriterListener* listener_ = nullptr;
@@ -344,6 +443,12 @@ protected:
                 fastrtps::rtps::RTPSWriter* writer,
                 const fastrtps::LivelinessLostStatus& status) override;
 
+        void on_reader_discovery(
+                fastrtps::rtps::RTPSWriter* writer,
+                fastrtps::rtps::ReaderDiscoveryInfo::DISCOVERY_STATUS reason,
+                const fastrtps::rtps::GUID_t& reader_guid,
+                const fastrtps::rtps::ReaderProxyData* reader_info) override;
+
         DataWriterImpl* data_writer_;
     }
     writer_listener_;
@@ -361,7 +466,10 @@ protected:
     PublicationMatchedStatus publication_matched_status_;
 
     //! The offered deadline missed status
-    fastrtps::OfferedDeadlineMissedStatus deadline_missed_status_;
+    OfferedDeadlineMissedStatus deadline_missed_status_;
+
+    //! The liveliness lost status
+    LivelinessLostStatus liveliness_lost_status_;
 
     //! The offered incompatible qos status
     OfferedIncompatibleQosStatus offered_incompatible_qos_status_;
@@ -384,6 +492,23 @@ protected:
 
     fastrtps::rtps::GUID_t guid_;
 
+    std::unique_ptr<ReaderFilterCollection> reader_filters_;
+
+    ReturnCode_t check_write_preconditions(
+            void* data,
+            const InstanceHandle_t& handle,
+            InstanceHandle_t& instance_handle);
+
+    ReturnCode_t check_instance_preconditions(
+            void* data,
+            const InstanceHandle_t& handle,
+            InstanceHandle_t& instance_handle);
+
+    InstanceHandle_t do_register_instance(
+            void* key,
+            const InstanceHandle_t instance_handle,
+            fastrtps::rtps::WriteParams& wparams);
+
     /**
      *
      * @param kind
@@ -476,6 +601,15 @@ protected:
     OfferedIncompatibleQosStatus& update_offered_incompatible_qos(
             PolicyMask incompatible_policies);
 
+    /*!
+     * @brief Updates liveliness lost status.
+     *
+     * @param[in] liveliness_lost_status Liveliness lost status coming from RTPS layer.
+     * @return Current liveliness lost status.
+     */
+    LivelinessLostStatus& update_liveliness_lost_status(
+            const fastrtps::LivelinessLostStatus& liveliness_lost_status);
+
     /**
      * Returns the most appropriate listener to handle the callback for the given status,
      * or nullptr if there is no appropriate listener.
@@ -488,6 +622,8 @@ protected:
             fastrtps::rtps::CacheChange_t* ch,
             const uint32_t& high_mark_for_frag);
 
+    std::shared_ptr<IChangePool> get_change_pool() const;
+
     std::shared_ptr<IPayloadPool> get_payload_pool();
 
     bool release_payload_pool();
@@ -533,6 +669,30 @@ protected:
             void* data,
             PayloadInfo_t& payload);
 
+    /**
+     * Remove internal filtering information about a reader.
+     * Called whenever a non-intra-process reader is unmatched.
+     *
+     * @param reader_guid  GUID of the reader that has been unmatched.
+     */
+    void remove_reader_filter(
+            const fastrtps::rtps::GUID_t& reader_guid);
+
+    /**
+     * Process filtering information for a reader.
+     * Called when a new reader is matched, and whenever the discovery information of a matched reader changes.
+     *
+     * @param reader_guid  The GUID of the reader for which the discovery information changed.
+     * @param reader_info  The reader's discovery information.
+     */
+    void process_reader_filter_info(
+            const fastrtps::rtps::GUID_t& reader_guid,
+            const fastrtps::rtps::ReaderProxyData& reader_info);
+
+    bool is_relevant(
+            const fastrtps::rtps::CacheChange_t& change,
+            const fastrtps::rtps::GUID_t& reader_guid) const override;
+
 };
 
 } /* namespace dds */
