@@ -309,7 +309,7 @@ ReturnCode_t SubscriberImpl::delete_datareader(
         {
             //First extract the reader from the maps to free the mutex
             DataReaderImpl* reader_impl = *dr_it;
-            if (!reader_impl->can_be_deleted())
+            if (!reader_impl->can_be_deleted(false))
             {
                 return ReturnCode_t::RETCODE_PRECONDITION_NOT_MET;
             }
