@@ -28,6 +28,7 @@
 #include <fastdds/dds/log/Log.hpp>
 #include <fastdds/rtps/attributes/ServerAttributes.h>
 #include <fastdds/rtps/builtin/BuiltinProtocols.h>
+#include <fastdds/rtps/builtin/discovery/endpoint/EDP.h>
 #include <fastdds/rtps/builtin/discovery/participant/PDPSimple.h>
 #include <fastdds/rtps/builtin/data/ParticipantProxyData.h>
 #include <fastdds/rtps/builtin/liveliness/WLP.h>
@@ -66,6 +67,8 @@ using UDPv4TransportDescriptor = fastdds::rtps::UDPv4TransportDescriptor;
 using TCPTransportDescriptor = fastdds::rtps::TCPTransportDescriptor;
 using SharedMemTransportDescriptor = fastdds::rtps::SharedMemTransportDescriptor;
 
+thread_local RTPSParticipantImpl* RTPSParticipantImpl::collections_mutex_owner_ = nullptr;
+
 static EntityId_t TrustedWriter(
         const EntityId_t& reader)
 {
@@ -129,6 +132,9 @@ RTPSParticipantImpl::RTPSParticipantImpl(
     , mp_ResourceSemaphore(new Semaphore(0))
     , IdCounter(0)
     , type_check_fn_(nullptr)
+    , client_override_(false)
+    , internal_metatraffic_locators_(false)
+    , internal_default_locators_(false)
 #if HAVE_SECURITY
     , m_security_manager(this)
 #endif // if HAVE_SECURITY
@@ -187,9 +193,9 @@ RTPSParticipantImpl::RTPSParticipantImpl(
                 TCPTransportDescriptor* pT = dynamic_cast<TCPTransportDescriptor*>(transportDescriptor.get());
                 if (pT && pT->listening_ports.empty())
                 {
-                    logError(RTPS_PARTICIPANT,
-                            "Participant " << m_att.getName() << " with GUID " << m_guid
-                                           << " tries to use discovery server over TCP without providing a proper listening port");
+                    logInfo(RTPS_PARTICIPANT,
+                            "Participant " << m_att.getName() << " with GUID " << m_guid <<
+                            " tries to use discovery server over TCP without providing a proper listening port.");
                 }
             }
         default:
@@ -244,6 +250,7 @@ RTPSParticipantImpl::RTPSParticipantImpl(
     if (m_att.builtin.metatrafficMulticastLocatorList.empty() && m_att.builtin.metatrafficUnicastLocatorList.empty())
     {
         get_default_metatraffic_locators();
+        internal_metatraffic_locators_ = true;
     }
     else
     {
@@ -297,6 +304,7 @@ RTPSParticipantImpl::RTPSParticipantImpl(
         //Default Unicast Locators in case they have not been provided
         /* INSERT DEFAULT UNICAST LOCATORS FOR THE PARTICIPANT */
         get_default_unicast_locators();
+        internal_default_locators_ = true;
         logInfo(RTPS_PARTICIPANT, m_att.getName() << " Created with NO default Unicast Locator List, adding Locators:"
                                                   << m_att.defaultUnicastLocatorList);
     }
@@ -319,10 +327,9 @@ RTPSParticipantImpl::RTPSParticipantImpl(
 
 #if HAVE_SECURITY
     // Start security
-    // TODO(Ricardo) Get returned value in future.
-    m_security_manager_initialized = m_security_manager.init(security_attributes_, PParam.properties,
-                    m_is_security_active);
-    if (!m_security_manager_initialized)
+    if (!m_security_manager.init(
+                security_attributes_,
+                PParam.properties))
     {
         // Participant will be deleted, no need to allocate buffers or create builtin endpoints
         return;
@@ -386,13 +393,10 @@ RTPSParticipantImpl::RTPSParticipantImpl(
 
 
 #if HAVE_SECURITY
-    if (m_is_security_active)
+    if (m_security_manager.is_security_active())
     {
-        m_is_security_active = m_security_manager.create_entities();
-        if (!m_is_security_active)
+        if (!m_security_manager.create_entities())
         {
-            // Participant will be deleted, no need to create builtin endpoints
-            m_security_manager_initialized = false;
             return;
         }
     }
@@ -400,6 +404,13 @@ RTPSParticipantImpl::RTPSParticipantImpl(
 
     mp_builtinProtocols = new BuiltinProtocols();
 
+    // Initialize builtin protocols
+    if (!mp_builtinProtocols->initBuiltinProtocols(this, m_att.builtin))
+    {
+        logError(RTPS_PARTICIPANT, "The builtin protocols were not correctly initialized");
+        return;
+    }
+
     if (c_GuidPrefix_Unknown != persistence_guid)
     {
         logInfo(RTPS_PARTICIPANT, "RTPSParticipant \"" << m_att.getName() << "\" with guidPrefix: " << m_guid.guidPrefix
@@ -410,6 +421,8 @@ RTPSParticipantImpl::RTPSParticipantImpl(
         logInfo(RTPS_PARTICIPANT,
                 "RTPSParticipant \"" << m_att.getName() << "\" with guidPrefix: " << m_guid.guidPrefix);
     }
+
+    initialized_ = true;
 }
 
 RTPSParticipantImpl::RTPSParticipantImpl(
@@ -420,16 +433,11 @@ RTPSParticipantImpl::RTPSParticipantImpl(
         RTPSParticipantListener* plisten)
     : RTPSParticipantImpl(domain_id, PParam, guidP, c_GuidPrefix_Unknown, par, plisten)
 {
-    client_override_ = false;
 }
 
 void RTPSParticipantImpl::enable()
 {
-    // Start builtin protocols
-    if (!mp_builtinProtocols->initBuiltinProtocols(this, m_att.builtin))
-    {
-        logError(RTPS_PARTICIPANT, "The builtin protocols were not correctly initialized");
-    }
+    mp_builtinProtocols->enable();
 
     //Start reception
     for (auto& receiver : m_receiverResourcelist)
@@ -440,12 +448,14 @@ void RTPSParticipantImpl::enable()
 
 void RTPSParticipantImpl::disable()
 {
-    // Ensure that other participants will not accidentally discover this one
-    if (mp_builtinProtocols && mp_builtinProtocols->mp_PDP)
+    if (nullptr == mp_builtinProtocols)
     {
-        mp_builtinProtocols->stopRTPSParticipantAnnouncement();
+        return;
     }
 
+    // Ensure that other participants will not accidentally discover this one
+    stopRTPSParticipantAnnouncement();
+
     // Disable Retries on Transports
     m_network_Factory.Shutdown();
 
@@ -456,15 +466,9 @@ void RTPSParticipantImpl::disable()
         block.disable();
     }
 
-    while (m_userReaderList.size() > 0)
-    {
-        deleteUserEndpoint(static_cast<Endpoint*>(*m_userReaderList.begin()));
-    }
+    deleteAllUserEndpoints();
 
-    while (m_userWriterList.size() > 0)
-    {
-        deleteUserEndpoint(static_cast<Endpoint*>(*m_userWriterList.begin()));
-    }
+    mp_event_thr.stop_thread();
 
     delete(mp_builtinProtocols);
     mp_builtinProtocols = nullptr;
@@ -747,11 +751,14 @@ bool RTPSParticipantImpl::create_writer(
         }
     }
 
-    std::lock_guard<std::recursive_mutex> guard(*mp_mutex);
-    m_allWriterList.push_back(SWriter);
-    if (!is_builtin)
     {
-        m_userWriterList.push_back(SWriter);
+        std::lock_guard<shared_mutex> _(endpoints_list_mutex);
+        m_allWriterList.push_back(SWriter);
+
+        if (!is_builtin)
+        {
+            m_userWriterList.push_back(SWriter);
+        }
     }
     *writer_out = SWriter;
 
@@ -882,11 +889,15 @@ bool RTPSParticipantImpl::create_reader(
         }
     }
 
-    std::lock_guard<std::recursive_mutex> guard(*mp_mutex);
-    m_allReaderList.push_back(SReader);
-    if (!is_builtin)
     {
-        m_userReaderList.push_back(SReader);
+        std::lock_guard<shared_mutex> _(endpoints_list_mutex);
+
+        m_allReaderList.push_back(SReader);
+
+        if (!is_builtin)
+        {
+            m_userReaderList.push_back(SReader);
+        }
     }
     *reader_out = SReader;
 
@@ -1005,6 +1016,56 @@ bool RTPSParticipantImpl::createWriter(
     return create_writer(WriterOut, param, entityId, isBuiltin, callback);
 }
 
+bool RTPSParticipantImpl::create_writer(
+        RTPSWriter** WriterOut,
+        WriterAttributes& watt,
+        const std::shared_ptr<IPayloadPool>& payload_pool,
+        const std::shared_ptr<IChangePool>& change_pool,
+        WriterHistory* hist,
+        WriterListener* listen,
+        const EntityId_t& entityId,
+        bool isBuiltin)
+{
+    if (!payload_pool)
+    {
+        logError(RTPS_PARTICIPANT, "Trying to create writer with null payload pool");
+        return false;
+    }
+
+    auto callback = [hist, listen, &payload_pool, &change_pool, this]
+                (const GUID_t& guid, WriterAttributes& watt, fastdds::rtps::FlowController* flow_controller,
+                    IPersistenceService* persistence, bool is_reliable) -> RTPSWriter*
+            {
+                if (is_reliable)
+                {
+                    if (persistence != nullptr)
+                    {
+                        return new StatefulPersistentWriter(this, guid, watt, payload_pool, change_pool,
+                                       flow_controller, hist, listen, persistence);
+                    }
+                    else
+                    {
+                        return new StatefulWriter(this, guid, watt, payload_pool, change_pool,
+                                       flow_controller, hist, listen);
+                    }
+                }
+                else
+                {
+                    if (persistence != nullptr)
+                    {
+                        return new StatelessPersistentWriter(this, guid, watt, payload_pool, change_pool,
+                                       flow_controller, hist, listen, persistence);
+                    }
+                    else
+                    {
+                        return new StatelessWriter(this, guid, watt, payload_pool, change_pool,
+                                       flow_controller, hist, listen);
+                    }
+                }
+            };
+    return create_writer(WriterOut, watt, entityId, isBuiltin, callback);
+}
+
 bool RTPSParticipantImpl::createReader(
         RTPSReader** ReaderOut,
         ReaderAttributes& param,
@@ -1094,9 +1155,7 @@ bool RTPSParticipantImpl::createReader(
 RTPSReader* RTPSParticipantImpl::find_local_reader(
         const GUID_t& reader_guid)
 {
-    // As this is only called from RTPSDomainImpl::find_local_reader, and it has
-    // the domain mutex taken, there is no need to take the participant mutex
-    // std::lock_guard<std::recursive_mutex> guard(*mp_mutex);
+    shared_lock<shared_mutex> _(endpoints_list_mutex);
 
     for (auto reader : m_allReaderList)
     {
@@ -1112,9 +1171,7 @@ RTPSReader* RTPSParticipantImpl::find_local_reader(
 RTPSWriter* RTPSParticipantImpl::find_local_writer(
         const GUID_t& writer_guid)
 {
-    // As this is only called from RTPSDomainImpl::find_local_reader, and it has
-    // the domain mutex taken, there is no need to take the participant mutex
-    // std::lock_guard<std::recursive_mutex> guard(*mp_mutex);
+    shared_lock<shared_mutex> _(endpoints_list_mutex);
 
     for (auto writer : m_allWriterList)
     {
@@ -1160,9 +1217,10 @@ bool RTPSParticipantImpl::registerWriter(
 bool RTPSParticipantImpl::registerReader(
         RTPSReader* reader,
         const TopicAttributes& topicAtt,
-        const ReaderQos& rqos)
+        const ReaderQos& rqos,
+        const fastdds::rtps::ContentFilterProperty* content_filter)
 {
-    return this->mp_builtinProtocols->addLocalReader(reader, topicAtt, rqos);
+    return this->mp_builtinProtocols->addLocalReader(reader, topicAtt, rqos, content_filter);
 }
 
 void RTPSParticipantImpl::update_attributes(
@@ -1170,21 +1228,18 @@ void RTPSParticipantImpl::update_attributes(
 {
     bool update_pdp = false;
     // Check if new interfaces have been added
-    if (patt.builtin.metatrafficMulticastLocatorList.empty() && patt.builtin.metatrafficUnicastLocatorList.empty())
+    if (internal_metatraffic_locators_)
     {
-        LocatorList_t metatraffic_multicast_locator_list = m_att.builtin.metatrafficMulticastLocatorList;
         LocatorList_t metatraffic_unicast_locator_list = m_att.builtin.metatrafficUnicastLocatorList;
-
         get_default_metatraffic_locators();
-
-        if (!(metatraffic_multicast_locator_list == m_att.builtin.metatrafficMulticastLocatorList) ||
-                !(metatraffic_unicast_locator_list == m_att.builtin.metatrafficUnicastLocatorList))
+        if (!(metatraffic_unicast_locator_list == m_att.builtin.metatrafficUnicastLocatorList))
         {
             update_pdp = true;
+            m_network_Factory.update_network_interfaces();
             logInfo(RTPS_PARTICIPANT, m_att.getName() << " updated its metatraffic locators");
         }
     }
-    if (patt.defaultUnicastLocatorList.empty() && patt.defaultMulticastLocatorList.empty())
+    if (internal_default_locators_)
     {
         LocatorList_t default_unicast_locator_list = m_att.defaultUnicastLocatorList;
         get_default_unicast_locators();
@@ -1197,23 +1252,37 @@ void RTPSParticipantImpl::update_attributes(
     }
 
     auto pdp = mp_builtinProtocols->mp_PDP;
+
     // Check if there are changes
     if (patt.builtin.discovery_config.m_DiscoveryServers != m_att.builtin.discovery_config.m_DiscoveryServers
-            || patt.userData != m_att.userData)
+            || patt.userData != m_att.userData
+            || update_pdp)
     {
         update_pdp = true;
-        // Check that the remote servers list is consistent: all the already known remote servers must be included in the
-        // list and only new remote servers can be added.
+        std::vector<GUID_t> modified_servers;
+        LocatorList_t modified_locators;
+
+        // Update RTPSParticipantAttributes member
+        m_att.userData = patt.userData;
+
+        // If there's no PDP don't process Discovery-related attributes.
+        if (!pdp)
+        {
+            return;
+        }
+
+        // Check that the remote servers list is consistent: all the already known remote servers must be included in
+        // the list and either new remote servers are added or remote server listening locator is modified.
         for (auto existing_server : m_att.builtin.discovery_config.m_DiscoveryServers)
         {
             bool contained = false;
-            bool locator_contained = false;
             for (auto incoming_server : patt.builtin.discovery_config.m_DiscoveryServers)
             {
                 if (existing_server.guidPrefix == incoming_server.guidPrefix)
                 {
                     for (auto incoming_locator : incoming_server.metatrafficUnicastLocatorList)
                     {
+                        bool locator_contained = false;
                         for (auto existing_locator : existing_server.metatrafficUnicastLocatorList)
                         {
                             if (incoming_locator == existing_locator)
@@ -1224,10 +1293,11 @@ void RTPSParticipantImpl::update_attributes(
                         }
                         if (!locator_contained)
                         {
-                            logWarning(RTPS_QOS_CHECK,
-                                    "Discovery Servers cannot add/modify their locators: " << incoming_locator <<
-                                    " has not been added")
-                            return;
+                            modified_servers.emplace_back(incoming_server.GetParticipant());
+                            modified_locators.push_back(incoming_locator);
+                            logInfo(RTPS_QOS_CHECK,
+                                    "DS Server: " << incoming_server.guidPrefix << " has modified its locators: "
+                                                  << incoming_locator << " being added")
                         }
                     }
                     contained = true;
@@ -1242,23 +1312,44 @@ void RTPSParticipantImpl::update_attributes(
             }
         }
 
-        // Update RTPSParticipantAttributes member
-        m_att.userData = patt.userData;
-
         {
-            std::unique_lock<std::recursive_mutex> lock(*pdp->getMutex());
+            std::lock_guard<std::recursive_mutex> lock(*pdp->getMutex());
 
             // Update user data
             auto local_participant_proxy_data = pdp->getLocalParticipantProxyData();
             local_participant_proxy_data->m_userData.data_vec(m_att.userData);
 
+            // Update metatraffic locators
+            for (auto locator : m_att.builtin.metatrafficMulticastLocatorList)
+            {
+                local_participant_proxy_data->metatraffic_locators.add_multicast_locator(locator);
+            }
+            for (auto locator : m_att.builtin.metatrafficUnicastLocatorList)
+            {
+                local_participant_proxy_data->metatraffic_locators.add_unicast_locator(locator);
+            }
+
+            // Update default locators
+            for (auto locator : m_att.defaultUnicastLocatorList)
+            {
+                local_participant_proxy_data->default_locators.add_unicast_locator(locator);
+            }
+
+            createSenderResources(m_att.builtin.metatrafficMulticastLocatorList);
+            createSenderResources(m_att.builtin.metatrafficUnicastLocatorList);
+            createSenderResources(m_att.defaultUnicastLocatorList);
+            if (!modified_locators.empty())
+            {
+                createSenderResources(modified_locators);
+            }
+
             // Update remote servers list
             if (m_att.builtin.discovery_config.discoveryProtocol == DiscoveryProtocol::CLIENT ||
                     m_att.builtin.discovery_config.discoveryProtocol == DiscoveryProtocol::SUPER_CLIENT ||
                     m_att.builtin.discovery_config.discoveryProtocol == DiscoveryProtocol::SERVER ||
                     m_att.builtin.discovery_config.discoveryProtocol == DiscoveryProtocol::BACKUP)
             {
-                // Add incoming servers iff we don't know about them already
+                // Add incoming servers iff we don't know about them already or the listening locator has been modified
                 for (auto incoming_server : patt.builtin.discovery_config.m_DiscoveryServers)
                 {
                     eprosima::fastdds::rtps::RemoteServerList_t::iterator server_it;
@@ -1267,6 +1358,16 @@ void RTPSParticipantImpl::update_attributes(
                     {
                         if (server_it->guidPrefix == incoming_server.guidPrefix)
                         {
+                            // Check if the listening locators have been modified
+                            for (auto guid : modified_servers)
+                            {
+                                if (guid == incoming_server.GetParticipant())
+                                {
+                                    server_it->metatrafficUnicastLocatorList =
+                                            incoming_server.metatrafficUnicastLocatorList;
+                                    break;
+                                }
+                            }
                             break;
                         }
                     }
@@ -1277,7 +1378,10 @@ void RTPSParticipantImpl::update_attributes(
                 }
 
                 // Update the servers list in builtin protocols
-                mp_builtinProtocols->m_DiscoveryServers = m_att.builtin.discovery_config.m_DiscoveryServers;
+                {
+                    std::unique_lock<eprosima::shared_mutex> disc_lock(mp_builtinProtocols->getDiscoveryMutex());
+                    mp_builtinProtocols->m_DiscoveryServers = m_att.builtin.discovery_config.m_DiscoveryServers;
+                }
 
                 // Notify PDPServer
                 if (m_att.builtin.discovery_config.discoveryProtocol == DiscoveryProtocol::SERVER ||
@@ -1285,6 +1389,11 @@ void RTPSParticipantImpl::update_attributes(
                 {
                     fastdds::rtps::PDPServer* pdp_server = static_cast<fastdds::rtps::PDPServer*>(pdp);
                     pdp_server->update_remote_servers_list();
+                    for (auto remote_server : modified_servers)
+                    {
+                        pdp_server->remove_remote_participant(remote_server,
+                                ParticipantDiscoveryInfo::DISCOVERY_STATUS::DROPPED_PARTICIPANT);
+                    }
                 }
                 // Notify PDPClient
                 else if (m_att.builtin.discovery_config.discoveryProtocol == DiscoveryProtocol::CLIENT ||
@@ -1292,6 +1401,11 @@ void RTPSParticipantImpl::update_attributes(
                 {
                     fastdds::rtps::PDPClient* pdp_client = static_cast<fastdds::rtps::PDPClient*>(pdp);
                     pdp_client->update_remote_servers_list();
+                    for (auto remote_server : modified_servers)
+                    {
+                        pdp_client->remove_remote_participant(remote_server,
+                                ParticipantDiscoveryInfo::DISCOVERY_STATUS::DROPPED_PARTICIPANT);
+                    }
                 }
             }
         }
@@ -1315,9 +1429,10 @@ bool RTPSParticipantImpl::updateLocalWriter(
 bool RTPSParticipantImpl::updateLocalReader(
         RTPSReader* reader,
         const TopicAttributes& topicAtt,
-        const ReaderQos& rqos)
+        const ReaderQos& rqos,
+        const fastdds::rtps::ContentFilterProperty* content_filter)
 {
-    return this->mp_builtinProtocols->updateLocalReader(reader, topicAtt, rqos);
+    return this->mp_builtinProtocols->updateLocalReader(reader, topicAtt, rqos, content_filter);
 }
 
 /*
@@ -1332,26 +1447,23 @@ bool RTPSParticipantImpl::existsEntityId(
         const EntityId_t& ent,
         EndpointKind_t kind) const
 {
+
+    auto check = [&ent](Endpoint* e)
+            {
+                return ent == e->getGuid().entityId;
+            };
+
+    shared_lock<shared_mutex> _(endpoints_list_mutex);
+
     if (kind == WRITER)
     {
-        for (std::vector<RTPSWriter*>::const_iterator it = m_userWriterList.begin(); it != m_userWriterList.end(); ++it)
-        {
-            if (ent == (*it)->getGuid().entityId)
-            {
-                return true;
-            }
-        }
+        return std::any_of(m_userWriterList.begin(), m_userWriterList.end(), check);
     }
     else
     {
-        for (std::vector<RTPSReader*>::const_iterator it = m_userReaderList.begin(); it != m_userReaderList.end(); ++it)
-        {
-            if (ent == (*it)->getGuid().entityId)
-            {
-                return true;
-            }
-        }
+        return std::any_of(m_userReaderList.begin(), m_userReaderList.end(), check);
     }
+
     return false;
 }
 
@@ -1557,7 +1669,7 @@ bool RTPSParticipantImpl::createReceiverResources(
 void RTPSParticipantImpl::createSenderResources(
         const LocatorList_t& locator_list)
 {
-    std::unique_lock<std::timed_mutex> lock(m_send_resources_mutex_);
+    std::lock_guard<std::timed_mutex> lock(m_send_resources_mutex_);
 
     for (auto it_loc = locator_list.begin(); it_loc != locator_list.end(); ++it_loc)
     {
@@ -1568,109 +1680,215 @@ void RTPSParticipantImpl::createSenderResources(
 void RTPSParticipantImpl::createSenderResources(
         const Locator_t& locator)
 {
-    std::unique_lock<std::timed_mutex> lock(m_send_resources_mutex_);
+    std::lock_guard<std::timed_mutex> lock(m_send_resources_mutex_);
 
     m_network_Factory.build_send_resources(send_resource_list_, locator);
 }
 
 bool RTPSParticipantImpl::deleteUserEndpoint(
-        Endpoint* p_endpoint)
+        const GUID_t& endpoint)
 {
-    m_receiverResourcelistMutex.lock();
-    for (auto it = m_receiverResourcelist.begin(); it != m_receiverResourcelist.end(); ++it)
+    if ( getGuid().guidPrefix != endpoint.guidPrefix)
     {
-        it->mp_receiver->removeEndpoint(p_endpoint);
+        return false;
     }
-    m_receiverResourcelistMutex.unlock();
 
     bool found = false, found_in_users = false;
+    Endpoint* p_endpoint = nullptr;
+
+    if (endpoint.entityId.is_writer())
     {
-        if (p_endpoint->getAttributes().endpointKind == WRITER)
+        std::lock_guard<shared_mutex> _(endpoints_list_mutex);
+
+        for (auto wit = m_userWriterList.begin(); wit != m_userWriterList.end(); ++wit)
         {
-            std::lock_guard<std::recursive_mutex> guard(*mp_mutex);
-            for (auto wit = m_userWriterList.begin(); wit != m_userWriterList.end(); ++wit)
+            if ((*wit)->getGuid().entityId == endpoint.entityId) //Found it
             {
-                if ((*wit)->getGuid().entityId == p_endpoint->getGuid().entityId) //Found it
-                {
-                    m_userWriterList.erase(wit);
-                    found_in_users = true;
-                    break;
-                }
+                m_userWriterList.erase(wit);
+                found_in_users = true;
+                break;
             }
-            for (auto wit = m_allWriterList.begin(); wit != m_allWriterList.end(); ++wit)
-            {
-                if ((*wit)->getGuid().entityId == p_endpoint->getGuid().entityId) //Found it
-                {
-                    m_allWriterList.erase(wit);
-                    found = true;
-                    break;
-                }
-            }
-        }
-        else
-        {
-            std::lock_guard<std::recursive_mutex> guard(*mp_mutex);
-            for (auto rit = m_userReaderList.begin(); rit != m_userReaderList.end(); ++rit)
-            {
-                if ((*rit)->getGuid().entityId == p_endpoint->getGuid().entityId) //Found it
-                {
-                    m_userReaderList.erase(rit);
-                    found_in_users = true;
-                    break;
-                }
-            }
-            for (auto rit = m_allReaderList.begin(); rit != m_allReaderList.end(); ++rit)
-            {
-                if ((*rit)->getGuid().entityId == p_endpoint->getGuid().entityId) //Found it
-                {
-                    m_allReaderList.erase(rit);
-                    found = true;
-                    break;
-                }
-            }
-        }
-        if (!found)
-        {
-            return false;
         }
 
-        //REMOVE FOR BUILTINPROTOCOLS
-        if (p_endpoint->getAttributes().endpointKind == WRITER)
+        for (auto wit = m_allWriterList.begin(); wit != m_allWriterList.end(); ++wit)
         {
-            if (found_in_users)
+            if ((*wit)->getGuid().entityId == endpoint.entityId) //Found it
             {
-                mp_builtinProtocols->removeLocalWriter(static_cast<RTPSWriter*>(p_endpoint));
+                p_endpoint = *wit;
+                m_allWriterList.erase(wit);
+                found = true;
+                break;
             }
-
-#if HAVE_SECURITY
-            if (p_endpoint->getAttributes().security_attributes().is_submessage_protected ||
-                    p_endpoint->getAttributes().security_attributes().is_payload_protected)
-            {
-                m_security_manager.unregister_local_writer(p_endpoint->getGuid());
-            }
-#endif // if HAVE_SECURITY
-        }
-        else
-        {
-            if (found_in_users)
-            {
-                mp_builtinProtocols->removeLocalReader(static_cast<RTPSReader*>(p_endpoint));
-            }
-
-#if HAVE_SECURITY
-            if (p_endpoint->getAttributes().security_attributes().is_submessage_protected ||
-                    p_endpoint->getAttributes().security_attributes().is_payload_protected)
-            {
-                m_security_manager.unregister_local_reader(p_endpoint->getGuid());
-            }
-#endif // if HAVE_SECURITY
         }
     }
-    //	std::lock_guard<std::recursive_mutex> guardEndpoint(*p_endpoint->getMutex());
+    else
+    {
+        std::lock_guard<shared_mutex> _(endpoints_list_mutex);
+
+        for (auto rit = m_userReaderList.begin(); rit != m_userReaderList.end(); ++rit)
+        {
+            if ((*rit)->getGuid().entityId == endpoint.entityId) //Found it
+            {
+                m_userReaderList.erase(rit);
+                found_in_users = true;
+                break;
+            }
+        }
+
+        for (auto rit = m_allReaderList.begin(); rit != m_allReaderList.end(); ++rit)
+        {
+            if ((*rit)->getGuid().entityId == endpoint.entityId) //Found it
+            {
+                p_endpoint = *rit;
+                m_allReaderList.erase(rit);
+                found = true;
+                break;
+            }
+        }
+    }
+
+    if (!found)
+    {
+        return false;
+    }
+
+    {
+        std::lock_guard<std::mutex> _(m_receiverResourcelistMutex);
+
+        for (auto& rb : m_receiverResourcelist)
+        {
+            auto receiver = rb.mp_receiver;
+            if (receiver)
+            {
+                receiver->removeEndpoint(p_endpoint);
+            }
+        }
+    }
+
+    //REMOVE FROM BUILTIN PROTOCOLS
+    if (p_endpoint->getAttributes().endpointKind == WRITER)
+    {
+        if (found_in_users)
+        {
+            mp_builtinProtocols->removeLocalWriter(static_cast<RTPSWriter*>(p_endpoint));
+        }
+
+#if HAVE_SECURITY
+        if (p_endpoint->getAttributes().security_attributes().is_submessage_protected ||
+                p_endpoint->getAttributes().security_attributes().is_payload_protected)
+        {
+            m_security_manager.unregister_local_writer(p_endpoint->getGuid());
+        }
+#endif // if HAVE_SECURITY
+    }
+    else
+    {
+        if (found_in_users)
+        {
+            mp_builtinProtocols->removeLocalReader(static_cast<RTPSReader*>(p_endpoint));
+        }
+
+#if HAVE_SECURITY
+        if (p_endpoint->getAttributes().security_attributes().is_submessage_protected ||
+                p_endpoint->getAttributes().security_attributes().is_payload_protected)
+        {
+            m_security_manager.unregister_local_reader(p_endpoint->getGuid());
+        }
+#endif // if HAVE_SECURITY
+    }
+
     delete(p_endpoint);
     return true;
 }
 
+void RTPSParticipantImpl::deleteAllUserEndpoints()
+{
+    std::vector<Endpoint*> tmp(0);
+
+    {
+        using namespace std;
+
+        lock_guard<shared_mutex> _(endpoints_list_mutex);
+
+        // move the collections to a local
+        tmp.resize(m_userWriterList.size() + m_userReaderList.size());
+        auto it = move(m_userWriterList.begin(), m_userWriterList.end(), tmp.begin());
+        it = move(m_userReaderList.begin(), m_userReaderList.end(), it);
+
+        // check we have copied all elements
+        assert(tmp.end() == it);
+
+        // now update the all collections by removing the user elements
+        sort(m_userWriterList.begin(), m_userWriterList.end());
+        sort(m_userReaderList.begin(), m_userReaderList.end());
+        sort(m_allWriterList.begin(), m_allWriterList.end());
+        sort(m_allReaderList.begin(), m_allReaderList.end());
+
+        vector<RTPSWriter*> writers;
+        set_difference(m_allWriterList.begin(), m_allWriterList.end(),
+                m_userWriterList.begin(), m_userWriterList.end(),
+                back_inserter(writers));
+        swap(writers, m_allWriterList);
+
+        vector<RTPSReader*> readers;
+        set_difference(m_allReaderList.begin(), m_allReaderList.end(),
+                m_userReaderList.begin(), m_userReaderList.end(),
+                back_inserter(readers));
+        swap(readers, m_allReaderList);
+
+        // remove dangling references
+        m_userWriterList.clear();
+        m_userReaderList.clear();
+    }
+
+    // unlink the transport receiver blocks from the endpoints
+    for ( auto endpoint : tmp)
+    {
+        std::lock_guard<std::mutex> _(m_receiverResourcelistMutex);
+
+        for (auto& rb : m_receiverResourcelist)
+        {
+            auto receiver = rb.mp_receiver;
+            if (receiver)
+            {
+                receiver->removeEndpoint(endpoint);
+            }
+        }
+    }
+
+    // Remove from builtin protocols
+    auto removeEndpoint = [this](EndpointKind_t kind, Endpoint* p)
+            {
+                return kind == WRITER
+               ? mp_builtinProtocols->removeLocalWriter((RTPSWriter*)p)
+               : mp_builtinProtocols->removeLocalReader((RTPSReader*)p);
+            };
+
+#if HAVE_SECURITY
+    bool (eprosima::fastrtps::rtps::security::SecurityManager::* unregister_endpoint[2])(
+            const GUID_t& writer_guid);
+    unregister_endpoint[WRITER] = &security::SecurityManager::unregister_local_writer;
+    unregister_endpoint[READER] = &security::SecurityManager::unregister_local_reader;
+#endif // if HAVE_SECURITY
+
+    for ( auto endpoint : tmp)
+    {
+        auto kind = endpoint->getAttributes().endpointKind;
+        removeEndpoint(kind, endpoint);
+
+#if HAVE_SECURITY
+        if (endpoint->getAttributes().security_attributes().is_submessage_protected ||
+                endpoint->getAttributes().security_attributes().is_payload_protected)
+        {
+            (m_security_manager.*unregister_endpoint[kind])(endpoint->getGuid());
+        }
+#endif // if HAVE_SECURITY
+
+        // remove the endpoints
+        delete(endpoint);
+    }
+}
+
 void RTPSParticipantImpl::normalize_endpoint_locators(
         EndpointAttributes& endpoint_att)
 {
@@ -1876,7 +2094,7 @@ bool RTPSParticipantImpl::pairing_remote_writer_with_local_reader_after_security
 bool RTPSParticipantImpl::is_security_enabled_for_writer(
         const WriterAttributes& writer_attributes)
 {
-    if (!is_security_initialized() || !is_secure())
+    if (!is_initialized() || !is_secure())
     {
         return false;
     }
@@ -1899,7 +2117,7 @@ bool RTPSParticipantImpl::is_security_enabled_for_writer(
 bool RTPSParticipantImpl::is_security_enabled_for_reader(
         const ReaderAttributes& reader_attributes)
 {
-    if (!is_security_initialized() || !is_secure())
+    if (!is_initialized() || !is_secure())
     {
         return false;
     }
@@ -2251,6 +2469,7 @@ bool RTPSParticipantImpl::register_in_writer(
 
     if ( GUID_t::unknown() == writer_guid )
     {
+        shared_lock<shared_mutex> _(endpoints_list_mutex);
         res = true;
         for ( auto writer : m_userWriterList)
         {
@@ -2277,6 +2496,7 @@ bool RTPSParticipantImpl::register_in_reader(
 
     if ( GUID_t::unknown() == reader_guid )
     {
+        shared_lock<shared_mutex> _(endpoints_list_mutex);
         res = true;
         for ( auto reader : m_userReaderList)
         {
@@ -2298,7 +2518,7 @@ bool RTPSParticipantImpl::register_in_reader(
 bool RTPSParticipantImpl::unregister_in_writer(
         std::shared_ptr<fastdds::statistics::IListener> listener)
 {
-    std::lock_guard<std::recursive_mutex> guard(*getParticipantMutex());
+    shared_lock<shared_mutex> _(endpoints_list_mutex);
     bool res = true;
 
     for ( auto writer : m_userWriterList)
@@ -2315,7 +2535,7 @@ bool RTPSParticipantImpl::unregister_in_writer(
 bool RTPSParticipantImpl::unregister_in_reader(
         std::shared_ptr<fastdds::statistics::IListener> listener)
 {
-    std::lock_guard<std::recursive_mutex> guard(*getParticipantMutex());
+    shared_lock<shared_mutex> _(endpoints_list_mutex);
     bool res = true;
 
     for ( auto reader : m_userReaderList)
