@@ -91,58 +91,75 @@ StatelessReader::StatelessReader(
 bool StatelessReader::matched_writer_add(
         const WriterProxyData& wdata)
 {
-    std::lock_guard<RecursiveTimedMutex> guard(mp_mutex);
-    for (const RemoteWriterInfo_t& writer : matched_writers_)
     {
-        if (writer.guid == wdata.guid())
+        std::unique_lock<RecursiveTimedMutex> guard(mp_mutex);
+        for (const RemoteWriterInfo_t& writer : matched_writers_)
         {
-            logWarning(RTPS_READER, "Attempting to add existing writer");
-            return false;
-        }
-    }
-
-    bool is_same_process = RTPSDomainImpl::should_intraprocess_between(m_guid, wdata.guid());
-    bool is_datasharing = !is_same_process && is_datasharing_compatible_with(wdata);
-
-    RemoteWriterInfo_t info;
-    info.guid = wdata.guid();
-    info.persistence_guid = wdata.persistence_guid();
-    info.has_manual_topic_liveliness = (MANUAL_BY_TOPIC_LIVELINESS_QOS == wdata.m_qos.m_liveliness.kind);
-    info.is_datasharing = is_datasharing;
-
-    if (is_datasharing)
-    {
-        if (datasharing_listener_->add_datasharing_writer(wdata.guid(),
-                m_att.durabilityKind == VOLATILE,
-                mp_history->m_att.maximumReservedCaches))
-        {
-            logInfo(RTPS_READER, "Writer Proxy " << wdata.guid() << " added to " << this->m_guid.entityId
-                                                 << " with data sharing");
-        }
-        else
-        {
-            logError(RTPS_READER, "Failed to add Writer Proxy " << wdata.guid()
-                                                                << " to " << this->m_guid.entityId
-                                                                << " with data sharing.");
-            return false;
+            if (writer.guid == wdata.guid())
+            {
+                logWarning(RTPS_READER, "Attempting to add existing writer");
+                if (nullptr != mp_listener)
+                {
+                    // call the listener without the lock taken
+                    guard.unlock();
+                    mp_listener->on_writer_discovery(this, WriterDiscoveryInfo::CHANGED_QOS_WRITER, wdata.guid(),
+                            &wdata);
+                }
+                return false;
+            }
         }
 
-    }
+        bool is_same_process = RTPSDomainImpl::should_intraprocess_between(m_guid, wdata.guid());
+        bool is_datasharing = !is_same_process && is_datasharing_compatible_with(wdata);
+
+        RemoteWriterInfo_t info;
+        info.guid = wdata.guid();
+        info.persistence_guid = wdata.persistence_guid();
+        info.has_manual_topic_liveliness = (MANUAL_BY_TOPIC_LIVELINESS_QOS == wdata.m_qos.m_liveliness.kind);
+        info.is_datasharing = is_datasharing;
 
-    if (matched_writers_.emplace_back(info) == nullptr)
-    {
-        logWarning(RTPS_READER, "No space to add writer " << wdata.guid() << " to reader " << m_guid);
         if (is_datasharing)
         {
-            datasharing_listener_->remove_datasharing_writer(wdata.guid());
+            if (datasharing_listener_->add_datasharing_writer(wdata.guid(),
+                    m_att.durabilityKind == VOLATILE,
+                    mp_history->m_att.maximumReservedCaches))
+            {
+                logInfo(RTPS_READER, "Writer Proxy " << wdata.guid() << " added to " << this->m_guid.entityId
+                                                     << " with data sharing");
+            }
+            else
+            {
+                logError(RTPS_READER, "Failed to add Writer Proxy " << wdata.guid()
+                                                                    << " to " << this->m_guid.entityId
+                                                                    << " with data sharing.");
+                return false;
+            }
+
+        }
+
+        if (matched_writers_.emplace_back(info) == nullptr)
+        {
+            logWarning(RTPS_READER, "No space to add writer " << wdata.guid() << " to reader " << m_guid);
+            if (is_datasharing)
+            {
+                datasharing_listener_->remove_datasharing_writer(wdata.guid());
+            }
+            return false;
+        }
+        logInfo(RTPS_READER, "Writer " << wdata.guid() << " added to reader " << m_guid);
+
+        add_persistence_guid(info.guid, info.persistence_guid);
+
+        m_acceptMessagesFromUnkownWriters = false;
+
+        // Intraprocess manages durability itself
+        if (is_datasharing && !is_same_process && m_att.durabilityKind != VOLATILE)
+        {
+            // simulate a notification to force reading of transient changes
+            // this has to be done after the writer is added to the matched_writers or the processing may fail
+            datasharing_listener_->notify(false);
         }
-        return false;
     }
-    logInfo(RTPS_READER, "Writer " << wdata.guid() << " added to reader " << m_guid);
-
-    add_persistence_guid(info.guid, info.persistence_guid);
-
-    m_acceptMessagesFromUnkownWriters = false;
 
     if (liveliness_lease_duration_ < c_TimeInfinite)
     {
@@ -160,12 +177,9 @@ bool StatelessReader::matched_writer_add(
         }
     }
 
-    // Intraprocess manages durability itself
-    if (is_datasharing && !is_same_process && m_att.durabilityKind != VOLATILE)
+    if (nullptr != mp_listener)
     {
-        // simulate a notification to force reading of transient changes
-        // this has to be done after the writer is added to the matched_writers or the processing may fail
-        datasharing_listener_->notify(false);
+        mp_listener->on_writer_discovery(this, WriterDiscoveryInfo::DISCOVERED_WRITER, wdata.guid(), &wdata);
     }
 
     return true;
@@ -175,11 +189,6 @@ bool StatelessReader::matched_writer_remove(
         const GUID_t& writer_guid,
         bool removed_by_lease)
 {
-    std::lock_guard<RecursiveTimedMutex> guard(mp_mutex);
-
-    //Remove cachechanges belonging to the unmatched writer
-    mp_history->writer_unmatched(writer_guid, get_last_notified(writer_guid));
-
     if (liveliness_lease_duration_ < c_TimeInfinite)
     {
         auto wlp = mp_RTPSParticipant->wlp();
@@ -196,26 +205,37 @@ bool StatelessReader::matched_writer_remove(
                     "Finite liveliness lease duration but WLP not enabled, cannot remove writer");
         }
     }
-
-    ResourceLimitedVector<RemoteWriterInfo_t>::iterator it;
-    for (it = matched_writers_.begin(); it != matched_writers_.end(); ++it)
     {
-        if (it->guid == writer_guid)
+        std::unique_lock<RecursiveTimedMutex> guard(mp_mutex);
+
+        //Remove cachechanges belonging to the unmatched writer
+        mp_history->writer_unmatched(writer_guid, get_last_notified(writer_guid));
+
+        ResourceLimitedVector<RemoteWriterInfo_t>::iterator it;
+        for (it = matched_writers_.begin(); it != matched_writers_.end(); ++it)
         {
-            logInfo(RTPS_READER, "Writer " << writer_guid << " removed from " << m_guid);
-
-            if (it->is_datasharing && datasharing_listener_->remove_datasharing_writer(writer_guid))
+            if (it->guid == writer_guid)
             {
-                logInfo(RTPS_READER, "Data sharing writer " << writer_guid << " removed from " << m_guid.entityId);
-                remove_changes_from(writer_guid, true);
-            }
+                logInfo(RTPS_READER, "Writer " << writer_guid << " removed from " << m_guid);
 
-            remove_persistence_guid(it->guid, it->persistence_guid, removed_by_lease);
-            matched_writers_.erase(it);
-            return true;
+                if (it->is_datasharing && datasharing_listener_->remove_datasharing_writer(writer_guid))
+                {
+                    logInfo(RTPS_READER, "Data sharing writer " << writer_guid << " removed from " << m_guid.entityId);
+                    remove_changes_from(writer_guid, true);
+                }
+
+                remove_persistence_guid(it->guid, it->persistence_guid, removed_by_lease);
+                matched_writers_.erase(it);
+                if (nullptr != mp_listener)
+                {
+                    // call the listener without lock
+                    guard.unlock();
+                    mp_listener->on_writer_discovery(this, WriterDiscoveryInfo::REMOVED_WRITER, writer_guid, nullptr);
+                }
+                return true;
+            }
         }
     }
-
     return false;
 }
 
@@ -245,17 +265,37 @@ bool StatelessReader::change_received(
         if (mp_history->received_change(change, 0))
         {
             auto payload_length = change->serializedPayload.length;
+            auto guid = change->writerGUID;
+            auto seq = change->sequenceNumber;
 
             Time_t::now(change->reader_info.receptionTimestamp);
-            update_last_notified(change->writerGUID, change->sequenceNumber);
+            SequenceNumber_t previous_seq = update_last_notified(change->writerGUID, change->sequenceNumber);
             ++total_unread_;
 
-            on_data_notify(change->writerGUID, change->sourceTimestamp);
+            on_data_notify(guid, change->sourceTimestamp);
 
-            if (getListener() != nullptr)
+            auto listener = getListener();
+            if (listener != nullptr)
             {
-                // WARNING! This method could destroy the change
-                getListener()->onNewCacheChangeAdded(this, change);
+                if (SequenceNumber_t{0, 0} != previous_seq)
+                {
+                    assert(previous_seq < seq);
+                    uint64_t tmp = (seq - previous_seq).to64long() - 1;
+                    int32_t lost_samples = tmp > static_cast<uint64_t>(std::numeric_limits<int32_t>::max()) ?
+                            std::numeric_limits<int32_t>::max() : static_cast<int32_t>(tmp);
+                    if (0 < lost_samples) // There are lost samples.
+                    {
+                        listener->on_sample_lost(this, lost_samples);
+                    }
+                }
+
+                // WARNING! These methods could destroy the change
+                bool notify_single = false;
+                listener->on_data_available(this, guid, seq, seq, notify_single);
+                if (notify_single)
+                {
+                    listener->onNewCacheChangeAdded(this, change);
+                }
             }
 
             new_notification_cv_.notify_all();
@@ -430,7 +470,9 @@ bool StatelessReader::processDataMsg(
             CacheChange_t* change_to_add = nullptr;
             if (!change_pool_->reserve_cache(change_to_add))
             {
-                logError(RTPS_MSG_IN, IDSTRING "Problem reserving CacheChange in reader: " << m_guid);
+                logWarning(RTPS_MSG_IN,
+                        IDSTRING "Reached the maximum number of samples allowed by this reader's QoS. Rejecting change for reader: " <<
+                        m_guid );
                 return false;
             }
 
@@ -601,7 +643,12 @@ bool StatelessReader::processDataFragMsg(
                 // If the change was completed, process it.
                 if (change_completed != nullptr)
                 {
-                    if (data_filter_ && !data_filter_->is_relevant(*change_completed, m_guid))
+                    // Temporarilly assign the inline qos while evaluating the data filter
+                    change_completed->inline_qos = incomingChange->inline_qos;
+                    bool filtered_out = data_filter_ && !data_filter_->is_relevant(*change_completed, m_guid);
+                    change_completed->inline_qos = SerializedPayload_t();
+
+                    if (filtered_out)
                     {
                         update_last_notified(change_completed->writerGUID, change_completed->sequenceNumber);
                         releaseCache(change_completed);
