@@ -88,6 +88,7 @@ WriterProxy::WriterProxy(
     , liveliness_kind_(AUTOMATIC_LIVELINESS_QOS)
     , locators_entry_(loc_alloc.max_unicast_locators, loc_alloc.max_multicast_locators)
     , is_datasharing_writer_(false)
+    , received_at_least_one_heartbeat_(false)
 {
     //Create Events
     ResourceEvent& event_manager = reader_->getRTPSParticipant()->getEventResource();
@@ -98,8 +99,7 @@ WriterProxy::WriterProxy(
             };
     auto acknack_lambda = [this]() -> bool
             {
-                perform_initial_ack_nack();
-                return false;
+                return perform_initial_ack_nack();
             };
 
     heartbeat_response_ = new TimedEvent(event_manager, heartbeat_lambda, 0);
@@ -141,6 +141,7 @@ void WriterProxy::start(
     is_datasharing_writer_ = is_datasharing;
     initial_acknack_->restart_timer();
     loaded_from_storage(initial_sequence);
+    received_at_least_one_heartbeat_ = false;
 }
 
 void WriterProxy::update(
@@ -206,7 +207,7 @@ void WriterProxy::missing_changes_update(
     }
 }
 
-void WriterProxy::lost_changes_update(
+int32_t WriterProxy::lost_changes_update(
         const SequenceNumber_t& seq_num)
 {
 #ifdef SHOULD_DEBUG_LINUX
@@ -214,12 +215,27 @@ void WriterProxy::lost_changes_update(
 #endif // SHOULD_DEBUG_LINUX
 
     logInfo(RTPS_READER, guid().entityId << ": up to seq_num: " << seq_num);
+    int32_t current_sample_lost = 0;
 
     // Check was not removed from container.
-    if (seq_num > changes_from_writer_low_mark_)
+    if (seq_num > (changes_from_writer_low_mark_ + 1))
     {
         // Remove all received changes with a sequence lower than seq_num
         ChangeIterator it = std::lower_bound(changes_received_.begin(), changes_received_.end(), seq_num);
+        if (!changes_received_.empty())
+        {
+            uint64_t tmp = (*changes_received_.begin()).to64long() - (changes_from_writer_low_mark_.to64long() + 1);
+            auto distance = std::distance(changes_received_.begin(), it);
+            tmp += seq_num.to64long() - (*changes_received_.begin()).to64long() - distance;
+            current_sample_lost = tmp > static_cast<uint64_t>(std::numeric_limits<int32_t>::max()) ?
+                    std::numeric_limits<int32_t>::max() : static_cast<int32_t>(tmp);
+        }
+        else
+        {
+            uint64_t tmp = seq_num.to64long() - (changes_from_writer_low_mark_.to64long() + 1);
+            current_sample_lost = tmp > static_cast<uint64_t>(std::numeric_limits<int32_t>::max()) ?
+                    std::numeric_limits<int32_t>::max() : static_cast<int32_t>(tmp);
+        }
         changes_received_.erase(changes_received_.begin(), it);
 
         // Update low mark
@@ -232,6 +248,8 @@ void WriterProxy::lost_changes_update(
         // Next could need to be removed.
         cleanup();
     }
+
+    return current_sample_lost;
 }
 
 bool WriterProxy::received_change_set(
@@ -354,46 +372,6 @@ const SequenceNumber_t WriterProxy::available_changes_max() const
     return changes_from_writer_low_mark_;
 }
 
-void WriterProxy::change_removed_from_history(
-        const SequenceNumber_t& seq_num)
-{
-#ifdef SHOULD_DEBUG_LINUX
-    assert(get_mutex_owner() == get_thread_id());
-#endif // SHOULD_DEBUG_LINUX
-
-    // Check sequence number is in the container, because it was not clean up.
-    if (seq_num <= changes_from_writer_low_mark_)
-    {
-        return;
-    }
-
-    ChangeIterator chit = changes_received_.find(seq_num);
-
-    (void)chit;
-
-    // Element must be in the container. In other case, bug.
-    assert(chit != changes_received_.end());
-
-    // Previously, it was asserted that the change couldn't be the first and should have RECEIVED
-    // status. As we only keep received changes now, status is already checked by the previous assert.
-    // It can now be the case that the change being removed is the first if there are missing changes
-    // in the (changes_from_writer_low_mark_, seq_num) range.
-
-    // We are removing a change that will not be notified to the user. This may be due to the following:
-    // a) history became full (either due to KEEP_LAST or RESOURCE_LIMITS)
-    // b) lifespan timer expired for seq_num
-    // Previously, change was marked as irrelevant.
-
-    // As this may imply that all changes with a lower sequence number will also be dropped,
-    // now that history is full, it may be interesting to act as if a heartbeat with an initial
-    // sequence of seq_num has been received, i.e. calling lost_changes_update(seq_num).
-    // If we don't do it, changes_received_ may grow above the limits stablished for it,
-    // thus causing undesired dynamic allocations.
-
-    // For case a) a call to lost_changes_update is done inside StatefulReader::change_received.
-    // For case b) it does not imply a dynamic allocation problem.
-}
-
 void WriterProxy::cleanup()
 {
     ChangeIterator chit = changes_received_.begin();
@@ -430,27 +408,26 @@ size_t WriterProxy::unknown_missing_changes_up_to(
     if (seq_num > changes_from_writer_low_mark_)
     {
         SequenceNumber_t first_missing = changes_from_writer_low_mark_ + 1;
-        SequenceNumber_t max_missing = std::min(seq_num, max_sequence_number_ + 1);
         SequenceNumberSet_t sns(first_missing);
         SequenceNumberDiff d_fun;
 
         for (SequenceNumber_t seq : changes_received_)
         {
-            seq = std::min(seq, max_missing);
+            seq = std::min(seq, seq_num);
             if (first_missing < seq)
             {
                 returnedValue += d_fun(seq, first_missing);
             }
             first_missing = seq + 1;
-            if (first_missing >= max_missing)
+            if (first_missing >= seq_num)
             {
                 break;
             }
         }
 
-        if (first_missing < max_missing)
+        if (first_missing < seq_num)
         {
-            returnedValue += d_fun(max_missing, first_missing);
+            returnedValue += d_fun(seq_num, first_missing);
         }
     }
 
@@ -487,23 +464,34 @@ SequenceNumber_t WriterProxy::next_cache_change_to_be_notified()
     return SequenceNumber_t::unknown();
 }
 
-void WriterProxy::perform_initial_ack_nack()
+bool WriterProxy::perform_initial_ack_nack()
 {
-    // Send initial NACK.
-    SequenceNumberSet_t sns(SequenceNumber_t(0, 0));
-    if (is_on_same_process_)
+    bool ret_value = false;
+
+    if (!is_datasharing_writer_)
     {
-        RTPSWriter* writer = RTPSDomainImpl::find_local_writer(guid());
-        if (writer)
+        // Send initial NACK.
+        SequenceNumberSet_t sns(SequenceNumber_t(0, 0));
+        if (is_on_same_process_)
         {
-            bool tmp;
-            writer->process_acknack(guid(), reader_->getGuid(), 1, SequenceNumberSet_t(), false, tmp);
+            RTPSWriter* writer = RTPSDomainImpl::find_local_writer(guid());
+            if (writer)
+            {
+                bool tmp;
+                writer->process_acknack(guid(), reader_->getGuid(), 1, SequenceNumberSet_t(), false, tmp);
+            }
+        }
+        else
+        {
+            if (0 == last_heartbeat_count_)
+            {
+                reader_->send_acknack(this, sns, this, false);
+                ret_value = true;
+            }
         }
     }
-    else
-    {
-        reader_->send_acknack(this, sns, this, false);
-    }
+
+    return ret_value;
 }
 
 void WriterProxy::perform_heartbeat_response()
@@ -518,7 +506,8 @@ bool WriterProxy::process_heartbeat(
         bool final_flag,
         bool liveliness_flag,
         bool disable_positive,
-        bool& assert_liveliness)
+        bool& assert_liveliness,
+        int32_t& current_sample_lost)
 {
 #ifdef SHOULD_DEBUG_LINUX
     assert(get_mutex_owner() == get_thread_id());
@@ -534,7 +523,7 @@ bool WriterProxy::process_heartbeat(
         // initial_acknack_->cancel_timer();
 
         last_heartbeat_count_ = count;
-        lost_changes_update(first_seq);
+        current_sample_lost = lost_changes_update(first_seq);
         missing_changes_update(last_seq);
         heartbeat_final_flag_.store(final_flag);
 
@@ -565,6 +554,12 @@ bool WriterProxy::process_heartbeat(
             assert_liveliness = liveliness_flag;
         }
 
+        if (!received_at_least_one_heartbeat_)
+        {
+            current_sample_lost = 0;
+            received_at_least_one_heartbeat_ = true;
+        }
+
         return true;
     }
 
