@@ -226,7 +226,7 @@ void UDPv6Transport::endpoint_to_locator(
 void UDPv6Transport::fill_local_ip(
         Locator& loc) const
 {
-    loc.kind = LOCATOR_KIND_UDPv6;
+    loc.kind = kind();
     IPLocator::setIPv6(loc, "::1");
 }
 
@@ -585,6 +585,12 @@ bool UDPv6Transport::compare_ips(
     return false;
 }
 
+void UDPv6Transport::update_network_interfaces()
+{
+    UDPTransportInterface::update_network_interfaces();
+    // TODO(jlbueno)
+}
+
 } // namespace rtps
 } // namespace fastrtps
 } // namespace eprosima
