@@ -81,9 +81,10 @@ ReaderProxy::ReaderProxy(
 bool ReaderProxy::rtps_is_relevant(
         CacheChange_t* change) const
 {
-    if (nullptr != writer_->reader_data_filter())
+    auto filter = writer_->reader_data_filter();
+    if (nullptr != filter)
     {
-        bool ret = writer_->reader_data_filter()->is_relevant(*change, guid());
+        bool ret = filter->is_relevant(*change, guid());
         logInfo(RTPS_READER_PROXY,
                 "Change " << change->instanceHandle << " is relevant for reader " << guid() << "? " << ret);
         return ret;
@@ -269,6 +270,7 @@ bool ReaderProxy::change_is_unsent(
         const SequenceNumber_t& seq_num,
         FragmentNumber_t& next_unsent_frag,
         SequenceNumber_t& gap_seq,
+        const SequenceNumber_t& min_seq,
         bool& need_reactivate_periodic_heartbeat) const
 {
     if (seq_num <= changes_low_mark_ || changes_for_reader_.empty())
@@ -303,6 +305,19 @@ bool ReaderProxy::change_is_unsent(
             if (prev != chit->getSequenceNumber())
             {
                 gap_seq = prev;
+
+                // Verify the calculated gap_seq in ReaderProxy is a real hole in the history.
+                if (gap_seq < min_seq) // Several samples of the hole are not really already available.
+                {
+                    if (min_seq < seq_num)
+                    {
+                        gap_seq = min_seq;
+                    }
+                    else
+                    {
+                        gap_seq = SequenceNumber_t::unknown();
+                    }
+                }
             }
         }
     }
@@ -408,7 +423,7 @@ bool ReaderProxy::requested_changes_set(
                             isSomeoneWasSetRequested = true;
                         }
                     }
-                    else if ((sit > min_seq_in_history) && (sit > changes_low_mark_))
+                    else if ((sit >= min_seq_in_history) && (sit > changes_low_mark_))
                     {
                         gap_builder.add(sit);
                     }
@@ -461,7 +476,7 @@ void ReaderProxy::from_unsent_to_status(
     {
         assert(changes_for_reader_.begin() == it);
         changes_for_reader_.erase(it);
-        changes_low_mark_ = seq_num;
+        acked_changes_set(seq_num + 1);
         return;
     }
 
@@ -562,6 +577,12 @@ void ReaderProxy::change_has_been_removed(
 
     // Element may not be in the container when marked as irrelevant.
     changes_for_reader_.erase(chit);
+
+    // When removing the next-to-be-acknowledged, we should auto-acknowledge it.
+    if ((changes_low_mark_ + 1) == seq_num)
+    {
+        acked_changes_set(seq_num + 1);
+    }
 }
 
 bool ReaderProxy::has_unacknowledged(
