@@ -341,9 +341,17 @@ XMLP_ret XMLParser::parseXMLAliasDynamicType(
         if (valueBuilder != nullptr)
         {
             const char* name = p_root->Attribute(NAME);
-            p_dynamictypebuilder_t typeBuilder =
-                    types::DynamicTypeBuilderFactory::get_instance()->create_alias_builder(valueBuilder, name);
-            XMLProfileManager::insertDynamicTypeByName(name, typeBuilder);
+            if (name != nullptr)
+            {
+                p_dynamictypebuilder_t typeBuilder =
+                        types::DynamicTypeBuilderFactory::get_instance()->create_alias_builder(valueBuilder, name);
+                XMLProfileManager::insertDynamicTypeByName(name, typeBuilder);
+            }
+            else
+            {
+                logError(XMLPARSER, "Error parsing alias type: No name attribute given.");
+                ret = XMLP_ret::XML_ERROR;
+            }
         }
         else
         {
@@ -981,7 +989,15 @@ p_dynamictypebuilder_t XMLParser::parseXMLMemberDynamicType(
         uint32_t length = types::MAX_ELEMENTS_COUNT;
         if (lengthStr != nullptr)
         {
-            length = static_cast<uint32_t>(std::stoi(lengthStr));
+            try
+            {
+                length = static_cast<uint32_t>(std::stoi(lengthStr));
+            }
+            catch (const std::exception&)
+            {
+                logError(XMLPARSER, "Error parsing member sequence length in line " << p_root->GetLineNum());
+                return nullptr;
+            }
         }
 
         if (!isArray)
@@ -1057,7 +1073,15 @@ p_dynamictypebuilder_t XMLParser::parseXMLMemberDynamicType(
         uint32_t length = types::MAX_ELEMENTS_COUNT;
         if (lengthStr != nullptr)
         {
-            length = static_cast<uint32_t>(std::stoi(lengthStr));
+            try
+            {
+                length = static_cast<uint32_t>(std::stoi(lengthStr));
+            }
+            catch (const std::exception&)
+            {
+                logError(XMLPARSER, "Error parsing map member sequence length in line " << p_root->GetLineNum())
+                return nullptr;
+            }
         }
 
         if (!isArray)
@@ -1336,12 +1360,13 @@ p_dynamictypebuilder_t XMLParser::parseXMLMemberDynamicType(
     {
         if (!isArray)
         {
-            logError(XMLPARSER, "Failed creating " << memberType << ": " << memberName);
+            logError(XMLPARSER, "Failed creating " << memberType << ": " << (memberName ? memberName : ""));
         }
         else
         {
-            logError(XMLPARSER, "Failed creating " << memberType << " array: " << memberName);
+            logError(XMLPARSER, "Failed creating " << memberType << " array: " << (memberName ? memberName : ""));
         }
+        return nullptr;
     }
 
     const char* memberTopicKey = p_root->Attribute(KEY);
