@@ -177,7 +177,7 @@ void XMLProfileManager::loadDefaultXMLFile()
     size = 2;
 
     // Try to load the default XML file if variable does not exist or is not set to '1'
-    if (!(getenv_s(&size, skip_xml, size, "SKIP_DEFAULT_XML_FILE") == 0 && skip_xml[0] == '1'))
+    if (!(getenv_s(&size, skip_xml, size, SKIP_DEFAULT_XML_FILE) == 0 && skip_xml[0] == '1'))
     {
         loadXMLFile(DEFAULT_FASTRTPS_PROFILES);
     }
@@ -188,7 +188,7 @@ void XMLProfileManager::loadDefaultXMLFile()
         loadXMLFile(file_path);
     }
 
-    const char* skip_xml = std::getenv("SKIP_DEFAULT_XML_FILE");
+    const char* skip_xml = std::getenv(SKIP_DEFAULT_XML_FILE);
 
     // Try to load the default XML file if variable does not exist or is not set to '1'
     if (!(skip_xml != nullptr && skip_xml[0] == '1'))
