@@ -16,6 +16,10 @@
  * @file PKIDH.cpp
  */
 
+// TODO This isn't a proper fix for compatibility with OpenSSL 3.0, but
+// suppresses the warnings until true OpenSSL 3.0 APIs can be used.
+#define OPENSSL_API_COMPAT 10101
+
 #include <security/authentication/PKIDH.h>
 #include <security/authentication/PKIIdentityHandle.h>
 #include <fastdds/rtps/security/logging/Logging.h>
@@ -258,27 +262,27 @@ static EVP_PKEY* load_private_key(
         SecurityException& exception,
         PKIDH& pkidh)
 {
+    EVP_PKEY* key = nullptr;
+
     if (file.size() >= 7 && file.compare(0, 7, "file://") == 0)
     {
-        return detail::FileProvider::load_private_key(certificate, file, password, exception);
+        key = detail::FileProvider::load_private_key(certificate, file, password, exception);
     }
     else if (file.size() >= 7 && file.compare(0, 7, "pkcs11:") == 0)
     {
-#if HAVE_LIBP11
         if (!pkidh.pkcs11_provider)
         {
             pkidh.pkcs11_provider.reset(new detail::Pkcs11Provider());
         }
-        return pkidh.pkcs11_provider->load_private_key(certificate, file, password, exception);
-#else  // HAVE_LIBP11
-        static_cast<void>(pkidh);
-        exception = _SecurityException_(std::string("PKCS11 URIs require libp11 ") + file);
-        return nullptr;
-#endif // HAVE_LIBP11
+
+        key = pkidh.pkcs11_provider->load_private_key(certificate, file, password, exception);
+    }
+    else
+    {
+        exception = _SecurityException_(std::string("Unsupported URI format ") + file);
     }
 
-    exception = _SecurityException_(std::string("Unsupported URI format ") + file);
-    return nullptr;
+    return key;
 }
 
 static bool store_certificate_in_buffer(
@@ -601,11 +605,21 @@ static EVP_PKEY* generate_dh_key(
         params = EVP_PKEY_new();
         if (params != nullptr)
         {
-            if (1 != EVP_PKEY_set1_DH(params, DH_get_2048_256()))
+            DH* dh = DH_get_2048_256();
+            if (dh != nullptr)
             {
-                exception = _SecurityException_("Cannot set default parameters: ");
-                EVP_PKEY_free(params);
-                return nullptr;
+#if IS_OPENSSL_1_1_1d
+                int dh_type = DH_get0_q(dh) == NULL ? EVP_PKEY_DH : EVP_PKEY_DHX;
+                if (EVP_PKEY_assign(params, dh_type, dh) <= 0)
+#else
+                if (EVP_PKEY_assign_DH(params, dh) <= 0)
+#endif // if IS_OPENSSL_1_1_1d
+                {
+                    exception = _SecurityException_("Cannot set default parameters: ");
+                    DH_free(dh);
+                    EVP_PKEY_free(params);
+                    return nullptr;
+                }
             }
         }
         else
@@ -775,6 +789,7 @@ static EVP_PKEY* generate_dh_peer_key(
                     else
                     {
                         exception = _SecurityException_("OpenSSL library cannot set dh in pkey");
+                        DH_free(dh);
                     }
 
                     EVP_PKEY_free(key);
@@ -875,15 +890,15 @@ static bool generate_challenge(
     return returnedValue;
 }
 
-static SharedSecretHandle* generate_sharedsecret(
+std::shared_ptr<SecretHandle> PKIDH::generate_sharedsecret(
         EVP_PKEY* private_key,
         EVP_PKEY* public_key,
-        SecurityException& exception)
+        SecurityException& exception) const
 {
     assert(private_key);
     assert(public_key);
 
-    SharedSecretHandle* handle = nullptr;
+    std::shared_ptr<SharedSecretHandle> handle;
     EVP_PKEY_CTX* ctx = EVP_PKEY_CTX_new(private_key, NULL);
 
     if (ctx != nullptr)
@@ -905,7 +920,8 @@ static SharedSecretHandle* generate_sharedsecret(
                         if (EVP_Digest(data.value().data(), length, md, NULL, EVP_sha256(), NULL))
                         {
                             data.value().assign(md, md + 32);
-                            handle = new SharedSecretHandle();
+                            handle = std::dynamic_pointer_cast<SharedSecretHandle>(
+                                get_shared_secret(SharedSecretHandle::nil_handle, exception));
                             (*handle)->data_.push_back(std::move(data));
                         }
                         else
@@ -940,7 +956,7 @@ static SharedSecretHandle* generate_sharedsecret(
         exception = _SecurityException_("OpenSSL library cannot allocate context");
     }
 
-    return handle;
+    return std::dynamic_pointer_cast<SecretHandle>(handle);
 }
 
 static bool generate_identity_token(
@@ -1030,7 +1046,7 @@ ValidationResult_t PKIDH::validate_local_identity(
         password = &empty_password;
     }
 
-    PKIIdentityHandle* ih = new PKIIdentityHandle();
+    PKIIdentityHandle* ih = &PKIIdentityHandle::narrow(*get_identity_handle(exception));
 
     (*ih)->store_ = load_identity_ca(*identity_ca, (*ih)->there_are_crls_, (*ih)->sn, (*ih)->algo,
                     exception);
@@ -1122,7 +1138,7 @@ ValidationResult_t PKIDH::validate_remote_identity(
         const IdentityHandle& local_identity_handle,
         const IdentityToken& remote_identity_token,
         const GUID_t& remote_participant_key,
-        SecurityException& /*exception*/)
+        SecurityException& exception)
 {
     assert(remote_identity_handle);
     assert(local_identity_handle.nil() == false);
@@ -1142,7 +1158,7 @@ ValidationResult_t PKIDH::validate_remote_identity(
         // dds.cert.algo
         const std::string* cert_algo = DataHolderHelper::find_property_value(remote_identity_token, "dds.cert.algo");
 
-        PKIIdentityHandle* rih = new PKIIdentityHandle();
+        PKIIdentityHandle* rih = &PKIIdentityHandle::narrow(*get_identity_handle(exception));
 
         (*rih)->sn = ca_sn ? *ca_sn : "";
         (*rih)->cert_sn_ = ""; // cert_sn ? *cert_sn : "";
@@ -1344,7 +1360,7 @@ ValidationResult_t PKIDH::begin_handshake_reply(
         return ValidationResult_t::VALIDATION_FAILED;
     }
 
-    // Check incomming handshake.
+    // Check incoming handshake.
     // Check c.id
     const std::vector<uint8_t>* cid = DataHolderHelper::find_binary_property_value(handshake_message_in, "c.id");
     if (cid == nullptr)
@@ -1770,7 +1786,7 @@ ValidationResult_t PKIDH::process_handshake_request(
         return ValidationResult_t::VALIDATION_FAILED;
     }
 
-    // Check incomming handshake.
+    // Check incoming handshake.
     // Check c.id
     const std::vector<uint8_t>* cid = DataHolderHelper::find_binary_property_value(handshake_message_in, "c.id");
     if (cid == nullptr)
@@ -2163,8 +2179,10 @@ ValidationResult_t PKIDH::process_handshake_request(
     {
         final_message.binary_properties().push_back(std::move(bproperty));
 
-        handshake_handle->sharedsecret_ = generate_sharedsecret(handshake_handle->dhkeys_, handshake_handle->peerkeys_,
-                        exception);
+        handshake_handle->sharedsecret_ =
+                std::dynamic_pointer_cast<SharedSecretHandle>(
+            generate_sharedsecret(handshake_handle->dhkeys_, handshake_handle->peerkeys_,
+            exception));
 
         if (handshake_handle->sharedsecret_ != nullptr)
         {
@@ -2202,7 +2220,7 @@ ValidationResult_t PKIDH::process_handshake_reply(
         return ValidationResult_t::VALIDATION_FAILED;
     }
 
-    // Check incomming handshake.
+    // Check incoming handshake.
 
     // challenge1 (mandatory)
     BinaryProperty* challenge1 = DataHolderHelper::find_binary_property(handshake_message_in, "challenge1");
@@ -2358,8 +2376,10 @@ ValidationResult_t PKIDH::process_handshake_reply(
         return ValidationResult_t::VALIDATION_FAILED;
     }
 
-    handshake_handle->sharedsecret_ = generate_sharedsecret(handshake_handle->dhkeys_, handshake_handle->peerkeys_,
-                    exception);
+    handshake_handle->sharedsecret_ =
+            std::dynamic_pointer_cast<SharedSecretHandle>(
+        generate_sharedsecret(handshake_handle->dhkeys_, handshake_handle->peerkeys_,
+        exception));
 
     if (handshake_handle->sharedsecret_ != nullptr)
     {
@@ -2377,20 +2397,27 @@ ValidationResult_t PKIDH::process_handshake_reply(
     return ValidationResult_t::VALIDATION_FAILED;
 }
 
-SharedSecretHandle* PKIDH::get_shared_secret(
+std::shared_ptr<SecretHandle> PKIDH::get_shared_secret(
         const HandshakeHandle& handshake_handle,
-        SecurityException& /*exception*/)
+        SecurityException& exception) const
 {
     const PKIHandshakeHandle& handshake = PKIHandshakeHandle::narrow(handshake_handle);
 
     if (!handshake.nil())
     {
-        SharedSecretHandle* sharedsecret = new SharedSecretHandle();
+        auto secret = get_shared_secret(SharedSecretHandle::nil_handle, exception);
+        auto sharedsecret = std::dynamic_pointer_cast<SharedSecretHandle>(secret);
         (*sharedsecret)->data_ = (*handshake->sharedsecret_)->data_;
-        return sharedsecret;
+        return secret;
     }
 
-    return nullptr;
+    // create ad hoc deleter because this object can only be created/release from the friend factory
+    auto p = new (std::nothrow) SharedSecretHandle;
+    return std::dynamic_pointer_cast<SecretHandle>(
+        std::shared_ptr<SharedSecretHandle>(p, [](SharedSecretHandle* p)
+        {
+            delete p;
+        }));
 }
 
 bool PKIDH::set_listener(
@@ -2439,6 +2466,12 @@ bool PKIDH::return_handshake_handle(
     return false;
 }
 
+IdentityHandle* PKIDH::get_identity_handle(
+        SecurityException&)
+{
+    return new (std::nothrow) PKIIdentityHandle();
+}
+
 bool PKIDH::return_identity_handle(
         IdentityHandle* identity_handle,
         SecurityException& /*exception*/)
@@ -2455,10 +2488,10 @@ bool PKIDH::return_identity_handle(
 }
 
 bool PKIDH::return_sharedsecret_handle(
-        SharedSecretHandle* sharedsecret_handle,
-        SecurityException& /*exception*/)
+        std::shared_ptr<SecretHandle>& sharedsecret_handle,
+        SecurityException& /*exception*/) const
 {
-    delete sharedsecret_handle;
+    sharedsecret_handle.reset();
     return true;
 }
 
