@@ -22,6 +22,7 @@
 
 #include <atomic>
 #include <condition_variable>
+#include <functional>
 #include <list>
 #include <string>
 
@@ -58,6 +59,9 @@ using eprosima::fastdds::rtps::UDPTransportDescriptor;
 using eprosima::fastdds::rtps::UDPv4TransportDescriptor;
 using eprosima::fastdds::rtps::UDPv6TransportDescriptor;
 
+using SampleLostStatusFunctor = std::function<void (const eprosima::fastdds::dds::SampleLostStatus&)>;
+using SampleRejectedStatusFunctor = std::function<void (const eprosima::fastdds::dds::SampleRejectedStatus&)>;
+
 template<class TypeSupport>
 class PubSubReader
 {
@@ -173,7 +177,7 @@ protected:
                 bool ret = false;
                 do
                 {
-                    reader_.receive_one(datareader, ret);
+                    reader_.receive(datareader, ret);
                 } while (ret);
             }
         }
@@ -231,6 +235,24 @@ protected:
             }
         }
 
+        void on_sample_lost(
+                eprosima::fastdds::dds::DataReader* datareader,
+                const eprosima::fastdds::dds::SampleLostStatus& status) override
+        {
+            (void)datareader;
+
+            reader_.set_sample_lost_status(status);
+        }
+
+        void on_sample_rejected(
+                eprosima::fastdds::dds::DataReader* datareader,
+                const eprosima::fastdds::dds::SampleRejectedStatus& status) override
+        {
+            (void)datareader;
+
+            reader_.set_sample_rejected_status(status);
+        }
+
         unsigned int missed_deadlines() const
         {
             return times_deadline_missed_;
@@ -288,6 +310,10 @@ public:
         , last_incompatible_qos_(eprosima::fastdds::dds::INVALID_QOS_POLICY_ID)
         , message_receive_count_(0)
     {
+        // Load default QoS to permit testing with external XML profile files.
+        DomainParticipantFactory::get_instance()->load_profiles();
+        participant_qos_ = DomainParticipantFactory::get_instance()->get_default_participant_qos();
+
         // Generate topic name
         if (!statistics)
         {
@@ -311,6 +337,9 @@ public:
         // By default, heartbeat period delay is 100 milliseconds.
         datareader_qos_.reliable_reader_qos().times.heartbeatResponseDelay.seconds = 0;
         datareader_qos_.reliable_reader_qos().times.heartbeatResponseDelay.nanosec = 100000000;
+
+        // By default don't check for overlapping
+        loan_sample_validation(false);
     }
 
     virtual ~PubSubReader()
@@ -323,6 +352,11 @@ public:
         return *datareader_;
     }
 
+    eprosima::fastdds::dds::Subscriber& get_native_subscriber() const
+    {
+        return *subscriber_;
+    }
+
     void init()
     {
         ASSERT_FALSE(initialized_);
@@ -456,7 +490,7 @@ public:
         bool ret = false;
         do
         {
-            receive_one(datareader_, ret);
+            receive(datareader_, ret);
         }
         while (ret);
 
@@ -669,7 +703,8 @@ public:
         return ret_value;
     }
 
-    void wait_writer_undiscovery()
+    void wait_writer_undiscovery(
+            unsigned int matched = 0)
     {
         std::unique_lock<std::mutex> lock(mutexDiscovery_);
 
@@ -677,7 +712,7 @@ public:
 
         cvDiscovery_.wait(lock, [&]()
                 {
-                    return matched_ == 0;
+                    return matched_ <= matched;
                 });
 
         std::cout << "Reader removal finished..." << std::endl;
@@ -717,10 +752,10 @@ public:
     }
 
     void incompatible_qos(
-            eprosima::fastdds::dds::OfferedIncompatibleQosStatus status)
+            eprosima::fastdds::dds::RequestedIncompatibleQosStatus status)
     {
         std::unique_lock<std::mutex> lock(incompatible_qos_mutex_);
-        times_incompatible_qos_++;
+        times_incompatible_qos_ += status.total_count_change;
         last_incompatible_qos_ = status.last_policy_id;
         incompatible_qos_cv_.notify_one();
     }
@@ -796,6 +831,18 @@ public:
         return *this;
     }
 
+    PubSubReader& loan_sample_validation(
+            bool validate = true)
+    {
+        receive_ = std::bind(
+            validate ? &PubSubReader::receive_samples : &PubSubReader::receive_one,
+            this,
+            std::placeholders::_1,
+            std::placeholders::_2);
+
+        return *this;
+    }
+
     /*** Function to change QoS ***/
     PubSubReader& reliability(
             const eprosima::fastrtps::ReliabilityQosPolicyKind kind)
@@ -1310,6 +1357,13 @@ public:
         return *this;
     }
 
+    PubSubReader& guid_prefix(
+            const eprosima::fastrtps::rtps::GuidPrefix_t& prefix)
+    {
+        participant_qos_.wire_protocol().prefix = prefix;
+        return *this;
+    }
+
     PubSubReader& participant_id(
             int32_t participantId)
     {
@@ -1547,13 +1601,57 @@ public:
         return datareader_guid_;
     }
 
-protected:
+    eprosima::fastrtps::rtps::InstanceHandle_t datareader_ihandle()
+    {
+        return eprosima::fastrtps::rtps::InstanceHandle_t(datareader_guid());
+    }
+
+    void set_sample_lost_status(
+            const eprosima::fastdds::dds::SampleLostStatus& status)
+    {
+        if (sample_lost_status_functor_)
+        {
+            sample_lost_status_functor_(status);
+        }
+    }
+
+    PubSubReader& sample_lost_status_functor(
+            SampleLostStatusFunctor functor)
+    {
+        sample_lost_status_functor_ = functor;
+        return *this;
+    }
+
+    void set_sample_rejected_status(
+            const eprosima::fastdds::dds::SampleRejectedStatus& status)
+    {
+        if (sample_rejected_status_functor_)
+        {
+            sample_rejected_status_functor_(status);
+        }
+    }
+
+    PubSubReader& sample_rejected_status_functor(
+            SampleRejectedStatusFunctor functor)
+    {
+        sample_rejected_status_functor_ = functor;
+        return *this;
+    }
 
     const eprosima::fastrtps::rtps::GUID_t& participant_guid() const
     {
         return participant_guid_;
     }
 
+    eprosima::fastdds::dds::SubscriptionMatchedStatus get_subscription_matched_status() const
+    {
+        eprosima::fastdds::dds::SubscriptionMatchedStatus status;
+        datareader_->get_subscription_matched_status(status);
+        return status;
+    }
+
+private:
+
     void receive_one(
             eprosima::fastdds::dds::DataReader* datareader,
             bool& returnedValue)
@@ -1576,7 +1674,8 @@ protected:
             ASSERT_LT(last_seq[seq_info], info.sample_identity.sequence_number());
             last_seq[seq_info] = info.sample_identity.sequence_number();
 
-            if (info.instance_state == eprosima::fastdds::dds::ALIVE_INSTANCE_STATE)
+            if (info.valid_data
+                    && info.instance_state == eprosima::fastdds::dds::ALIVE_INSTANCE_STATE)
             {
                 auto it = std::find(total_msgs_.begin(), total_msgs_.end(), data);
                 ASSERT_NE(it, total_msgs_.end());
@@ -1588,6 +1687,72 @@ protected:
         }
     }
 
+    void receive_samples(
+            eprosima::fastdds::dds::DataReader* datareader,
+            bool& returnedValue)
+    {
+        eprosima::fastdds::dds::LoanableSequence<type> datas;
+        eprosima::fastdds::dds::SampleInfoSeq infos;
+        returnedValue = true;
+
+        ReturnCode_t success = take_ ?
+                datareader->take(datas, infos) :
+                datareader->read(datas, infos);
+
+        if (!success)
+        {
+            returnedValue = false;
+            return;
+        }
+
+        // Traverse the collection
+        std::unique_lock<std::mutex> lock(mutex_);
+        for (int32_t i = 0; i < datas.length(); ++i)
+        {
+            type& data = datas[i];
+            eprosima::fastdds::dds::SampleInfo& info = infos[i];
+
+            // Check order of changes.
+            LastSeqInfo seq_info{ info.instance_handle, info.sample_identity.writer_guid() };
+            ASSERT_LT(last_seq[seq_info], info.sample_identity.sequence_number());
+            last_seq[seq_info] = info.sample_identity.sequence_number();
+
+            if (info.valid_data
+                    && info.instance_state == eprosima::fastdds::dds::ALIVE_INSTANCE_STATE)
+            {
+                // Validate the sample
+                bool valid_sample = datareader->is_sample_valid(&data, &info);
+
+                EXPECT_TRUE(valid_sample) << "sample "
+                                          << info.sample_identity.sequence_number() << " was overlapped.";
+
+                if (valid_sample)
+                {
+                    auto it = std::find(total_msgs_.begin(), total_msgs_.end(), data);
+                    ASSERT_NE(it, total_msgs_.end());
+                    total_msgs_.erase(it);
+                    ++current_processed_count_;
+                    default_receive_print<type>(data);
+                    cv_.notify_one();
+                }
+            }
+        }
+
+        datareader->return_loan(datas, infos);
+    }
+
+    //! functor to check which API to retrieve samples
+    std::function<void (eprosima::fastdds::dds::DataReader* datareader, bool&)> receive_;
+
+protected:
+
+    void receive(
+            eprosima::fastdds::dds::DataReader* datareader,
+            bool& returnedValue)
+    {
+        receive_(datareader, std::ref(returnedValue));
+    }
+
     void participant_matched()
     {
         std::unique_lock<std::mutex> lock(mutexDiscovery_);
@@ -1711,6 +1876,11 @@ protected:
     std::condition_variable message_receive_cv_;
     //! Number of messages received but not yet processed by the application
     std::atomic<size_t> message_receive_count_;
+
+    //! Functor called when called SampleLostStatus listener.
+    SampleLostStatusFunctor sample_lost_status_functor_;
+    //! Functor called when called SampleRejectedStatus listener.
+    SampleRejectedStatusFunctor sample_rejected_status_functor_;
 };
 
 template<class TypeSupport>
@@ -1843,12 +2013,10 @@ protected:
                 if (status.alive_count_change == 1)
                 {
                     reader_.liveliness_recovered();
-
                 }
                 else if (status.not_alive_count_change == 1)
                 {
                     reader_.liveliness_lost();
-
                 }
             }
 
@@ -1865,11 +2033,25 @@ protected:
                     bool ret = false;
                     do
                     {
-                        reader_.receive_one(reader_.datareader_, ret);
+                        reader_.receive(reader_.datareader_, ret);
                     } while (ret);
                 }
             }
 
+            if (triggered_statuses.is_active(eprosima::fastdds::dds::StatusMask::sample_lost()))
+            {
+                eprosima::fastdds::dds::SampleLostStatus status;
+                ASSERT_EQ(ReturnCode_t::RETCODE_OK, reader_.datareader_->get_sample_lost_status(status));
+                reader_.set_sample_lost_status(status);
+            }
+
+            if (triggered_statuses.is_active(eprosima::fastdds::dds::StatusMask::sample_rejected()))
+            {
+                eprosima::fastdds::dds::SampleRejectedStatus status;
+                ASSERT_EQ(ReturnCode_t::RETCODE_OK, reader_.datareader_->get_sample_rejected_status(status));
+                reader_.set_sample_rejected_status(status);
+            }
+
             // We also have to process the subscriber
             triggered_statuses = reader_.subscriber_->get_status_changes();
             triggered_statuses &= condition->get_enabled_statuses();
@@ -1887,7 +2069,7 @@ protected:
                     bool ret = false;
                     do
                     {
-                        reader_.receive_one(reader_.datareader_, ret);
+                        reader_.receive(reader_.datareader_, ret);
                     } while (ret);
                 }
             }
