@@ -271,6 +271,10 @@ public:
         , unauthorized_(0)
 #endif // if HAVE_SECURITY
     {
+        // Load default QoS to permit testing with external XML profile files.
+        DomainParticipantFactory::get_instance()->load_profiles();
+        participant_qos_ = DomainParticipantFactory::get_instance()->get_default_participant_qos();
+
         // Generate topic name
         std::ostringstream t;
         t << topic_name << "_" << asio::ip::host_name() << "_" << GET_PID();
@@ -599,7 +603,8 @@ public:
         return ret_value;
     }
 
-    void wait_reader_undiscovery()
+    void wait_reader_undiscovery(
+            unsigned int matched = 0)
     {
         std::unique_lock<std::mutex> lock(mutexDiscovery_);
 
@@ -607,7 +612,7 @@ public:
 
         cv_.wait(lock, [&]()
                 {
-                    return matched_ == 0;
+                    return matched_ <= matched;
                 });
 
         std::cout << "Writer removal finished..." << std::endl;
@@ -686,8 +691,13 @@ public:
     bool waitForAllAcked(
             const std::chrono::duration<_Rep, _Period>& max_wait)
     {
+        auto nsecs = std::chrono::duration_cast<std::chrono::nanoseconds>(max_wait);
+        auto secs = std::chrono::duration_cast<std::chrono::seconds>(nsecs);
+        nsecs -= secs;
+        eprosima::fastrtps::Duration_t timeout {static_cast<int32_t>(secs.count()),
+                                                static_cast<uint32_t>(nsecs.count())};
         return (ReturnCode_t::RETCODE_OK ==
-               datawriter_->wait_for_acknowledgments(eprosima::fastrtps::Time_t((int32_t)max_wait.count(), 0)));
+               datawriter_->wait_for_acknowledgments(timeout));
     }
 
     template<class _Rep,
@@ -698,9 +708,13 @@ public:
             const eprosima::fastrtps::rtps::InstanceHandle_t& instance_handle,
             const std::chrono::duration<_Rep, _Period>& max_wait)
     {
+        auto nsecs = std::chrono::duration_cast<std::chrono::nanoseconds>(max_wait);
+        auto secs = std::chrono::duration_cast<std::chrono::seconds>(nsecs);
+        nsecs -= secs;
+        eprosima::fastrtps::Duration_t timeout {static_cast<int32_t>(secs.count()),
+                                                static_cast<uint32_t>(nsecs.count())};
         return (ReturnCode_t::RETCODE_OK ==
-               datawriter_->wait_for_acknowledgments(data, instance_handle,
-               eprosima::fastrtps::Time_t(static_cast<int32_t>(max_wait.count()), 0)));
+               datawriter_->wait_for_acknowledgments(data, instance_handle, timeout));
     }
 
     void block_until_discover_topic(
@@ -729,6 +743,11 @@ public:
                 });
     }
 
+    eprosima::fastdds::dds::DataWriterQos& qos()
+    {
+        return datawriter_qos_;
+    }
+
     PubSubWriter& deactivate_status_listener(
             eprosima::fastdds::dds::StatusMask mask)
     {
@@ -1339,6 +1358,11 @@ public:
         return datawriter_guid_;
     }
 
+    eprosima::fastrtps::rtps::InstanceHandle_t datawriter_ihandle()
+    {
+        return eprosima::fastrtps::rtps::InstanceHandle_t(datawriter_guid());
+    }
+
     bool update_partition(
             const std::string& partition)
     {
@@ -1390,6 +1414,13 @@ public:
         return status;
     }
 
+    eprosima::fastdds::dds::PublicationMatchedStatus get_publication_matched_status() const
+    {
+        eprosima::fastdds::dds::PublicationMatchedStatus status;
+        datawriter_->get_publication_matched_status(status);
+        return status;
+    }
+
     void set_xml_filename(
             const std::string& name)
     {
