@@ -808,6 +808,42 @@ TEST_P(PubSubBasic, ReliableTransientLocalTwoWritersConsecutives)
     }
 }
 
+// Regression test for redmine issue #14346
+TEST_P(PubSubBasic, ReliableHelloworldLateJoinersStress)
+{
+    if (enable_datasharing)
+    {
+        GTEST_SKIP() << "Data-sharing needs data reception for acknowledgement";
+    }
+
+    constexpr unsigned int num_iterations = 10;
+
+    PubSubWriter<HelloWorldPubSubType> writer(TEST_TOPIC_NAME);
+    writer.history_depth(10).init();
+    ASSERT_TRUE(writer.isInitialized());
+
+    std::vector<std::unique_ptr<PubSubReader<HelloWorldPubSubType>>> readers;
+
+    for (unsigned int i = 0; i < num_iterations; ++i)
+    {
+        readers.emplace_back(new PubSubReader<HelloWorldPubSubType>(TEST_TOPIC_NAME));
+        const auto& new_reader = readers.back();
+
+        new_reader->reliability(eprosima::fastrtps::RELIABLE_RELIABILITY_QOS)
+                .durability_kind(eprosima::fastrtps::TRANSIENT_LOCAL_DURABILITY_QOS)
+                .history_depth(10)
+                .init();
+        ASSERT_TRUE(new_reader->isInitialized()) << " on iteration " << i;
+        new_reader->wait_discovery();
+        writer.wait_discovery(i + 1);
+
+        auto data = default_helloworld_data_generator();
+        writer.send(data);
+        EXPECT_TRUE(data.empty()) << " on iteration " << i;
+        ASSERT_TRUE(writer.waitForAllAcked(std::chrono::seconds(5))) << " on iteration " << i;
+    }
+}
+
 /*
  * Check that setting FASTDDS_ENVIRONMENT_FILE to an unexisting file issues 1 logWarning
  */
