@@ -53,6 +53,12 @@ public:
     typedef TypeSupport type_support;
     typedef typename type_support::type type;
 
+    using OnWriterDiscoveryFunctor = std::function<void (
+                        eprosima::fastrtps::rtps::WriterDiscoveryInfo::DISCOVERY_STATUS,
+                        const eprosima::fastrtps::rtps::GUID_t&,
+                        const eprosima::fastrtps::rtps::WriterProxyData*
+                        )>;
+
 private:
 
     class Listener : public eprosima::fastrtps::rtps::ReaderListener
@@ -93,6 +99,15 @@ private:
             }
         }
 
+        void on_writer_discovery(
+                eprosima::fastrtps::rtps::RTPSReader* reader,
+                eprosima::fastrtps::rtps::WriterDiscoveryInfo::DISCOVERY_STATUS reason,
+                const eprosima::fastrtps::rtps::GUID_t& writer_guid,
+                const eprosima::fastrtps::rtps::WriterProxyData* writer_info) override
+        {
+            reader_.on_writer_discovery(reader, reason, writer_guid, writer_info);
+        }
+
     private:
 
         Listener& operator =(
@@ -170,9 +185,17 @@ public:
             return;
         }
 
-        ASSERT_EQ(participant_->registerReader(reader_, topic_attr_, reader_qos_), true);
+        initialized_ = participant_->registerReader(reader_, topic_attr_, reader_qos_, content_filter_property_);
+    }
 
-        initialized_ = true;
+    void update()
+    {
+        if (reader_ == nullptr)
+        {
+            return;
+        }
+
+        initialized_ = participant_->updateReader(reader_, topic_attr_, reader_qos_, content_filter_property_);
     }
 
     bool isInitialized() const
@@ -463,6 +486,39 @@ public:
 
 #endif // if HAVE_SQLITE3
 
+    RTPSWithRegistrationReader& user_data(
+            const std::vector<eprosima::fastrtps::rtps::octet>& user_data)
+    {
+        reader_qos_.m_userData = user_data;
+        return *this;
+    }
+
+    RTPSWithRegistrationReader& set_on_writer_discovery(
+            const OnWriterDiscoveryFunctor& functor)
+    {
+        on_writer_discovery_functor = functor;
+        return *this;
+    }
+
+    RTPSWithRegistrationReader& partitions(
+            std::vector<std::string>& partitions)
+    {
+        reader_qos_.m_partition.setNames(partitions);
+        return *this;
+    }
+
+    RTPSWithRegistrationReader& content_filter_property(
+            const eprosima::fastdds::rtps::ContentFilterProperty& content_filter_property)
+    {
+        content_filter_property_ = &content_filter_property;
+        return *this;
+    }
+
+    const eprosima::fastrtps::rtps::GUID_t& guid() const
+    {
+        return reader_->getGuid();
+    }
+
 private:
 
     void receive_one(
@@ -515,6 +571,20 @@ private:
         }
     }
 
+    void on_writer_discovery(
+            eprosima::fastrtps::rtps::RTPSReader* reader,
+            eprosima::fastrtps::rtps::WriterDiscoveryInfo::DISCOVERY_STATUS reason,
+            const eprosima::fastrtps::rtps::GUID_t& writer_guid,
+            const eprosima::fastrtps::rtps::WriterProxyData* writer_info)
+    {
+        ASSERT_EQ(reader_, reader);
+
+        if (on_writer_discovery_functor)
+        {
+            on_writer_discovery_functor(reason, writer_guid, writer_info);
+        }
+    }
+
     RTPSWithRegistrationReader& operator =(
             const RTPSWithRegistrationReader&) = delete;
 
@@ -539,6 +609,8 @@ private:
     type_support type_;
     std::shared_ptr<eprosima::fastrtps::rtps::IPayloadPool> payload_pool_;
     bool has_payload_pool_ = false;
+    OnWriterDiscoveryFunctor on_writer_discovery_functor;
+    const eprosima::fastdds::rtps::ContentFilterProperty* content_filter_property_ = nullptr;
 };
 
 #endif // _TEST_BLACKBOX_RTPSWITHREGISTRATIONREADER_HPP_
