@@ -658,8 +658,8 @@ TEST(DataWriterTests, Write)
 
     FooType data;
     data.message("HelloWorld");
-    ASSERT_TRUE(datawriter->write(&data, fastrtps::rtps::c_InstanceHandle_Unknown) ==
-            ReturnCode_t::RETCODE_OK);
+    ASSERT_FALSE(datawriter->write(nullptr, HANDLE_NIL) == ReturnCode_t::RETCODE_OK);
+    ASSERT_TRUE(datawriter->write(&data, HANDLE_NIL) == ReturnCode_t::RETCODE_OK);
     ASSERT_TRUE(datawriter->write(&data, participant->get_instance_handle()) ==
             ReturnCode_t::RETCODE_PRECONDITION_NOT_MET);
 
@@ -669,6 +669,48 @@ TEST(DataWriterTests, Write)
     ASSERT_TRUE(DomainParticipantFactory::get_instance()->delete_participant(participant) == ReturnCode_t::RETCODE_OK);
 }
 
+TEST(DataWriterTests, WriteWithTimestamp)
+{
+    DomainParticipant* participant =
+            DomainParticipantFactory::get_instance()->create_participant(0, PARTICIPANT_QOS_DEFAULT);
+    ASSERT_NE(participant, nullptr);
+
+    Publisher* publisher = participant->create_publisher(PUBLISHER_QOS_DEFAULT);
+    ASSERT_NE(publisher, nullptr);
+
+    TypeSupport type(new TopicDataTypeMock());
+    type.register_type(participant);
+
+    Topic* topic = participant->create_topic("footopic", type.get_type_name(), TOPIC_QOS_DEFAULT);
+    ASSERT_NE(topic, nullptr);
+
+    DataWriter* datawriter = publisher->create_datawriter(topic, DATAWRITER_QOS_DEFAULT);
+    ASSERT_NE(datawriter, nullptr);
+
+    eprosima::fastrtps::Time_t ts{ 0, 1 };
+
+    FooType data;
+    data.message("HelloWorld");
+
+    // 1. Calling write with nullptr data returns RETCODE_BAD_PARAMETER
+    ASSERT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, datawriter->write_w_timestamp(nullptr, HANDLE_NIL, ts));
+    // 2. Calling write with an invalid timestamps returns RETCODE_BAD_PARAMETER
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER,
+            datawriter->write_w_timestamp(&data, HANDLE_NIL, fastrtps::c_TimeInfinite));
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER,
+            datawriter->write_w_timestamp(&data, HANDLE_NIL, fastrtps::c_TimeInvalid));
+    // 3. Calling write with a wrong instance handle returns RETCODE_PRECONDITION_NOT_MET
+    ASSERT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET,
+            datawriter->write_w_timestamp(&data, participant->get_instance_handle(), ts));
+    // 4. Correct case
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, datawriter->write_w_timestamp(&data, HANDLE_NIL, ts));
+
+    ASSERT_TRUE(publisher->delete_datawriter(datawriter) == ReturnCode_t::RETCODE_OK);
+    ASSERT_TRUE(participant->delete_topic(topic) == ReturnCode_t::RETCODE_OK);
+    ASSERT_TRUE(participant->delete_publisher(publisher) == ReturnCode_t::RETCODE_OK);
+    ASSERT_TRUE(DomainParticipantFactory::get_instance()->delete_participant(participant) == ReturnCode_t::RETCODE_OK);
+}
+
 void set_listener_test (
         DataWriter* writer,
         DataWriterListener* listener,
@@ -753,15 +795,17 @@ TEST(DataWriterTests, TerminateWithoutDestroyingWriter)
 }
 
 /**
- * This test checks unregister_instance API
+ * Create two disabled data writer objects, one for a non-keyed topic, and another one for a keyed topic.
+ *
+ * @param [out] datawriter           Pointer to the data writer created for the non-keyed topic.
+ * @param [out] instance_datawriter  Pointer to the data writer created for the keyed topic.
+ * @param [out] keyed_type_support   Optionally written with the type support of the keyed topic.
  */
-TEST(DataWriterTests, UnregisterInstance)
+static void create_writers_for_instance_test(
+        DataWriter*& datawriter,
+        DataWriter*& instance_datawriter,
+        TypeSupport* keyed_type_support = nullptr)
 {
-    // Test parameters
-    InstanceHandle_t handle;
-    InstanceFooType data;
-    data.message("HelloWorld");
-
     // Create participant
     DomainParticipant* participant =
             DomainParticipantFactory::get_instance()->create_participant(0, PARTICIPANT_QOS_DEFAULT);
@@ -786,11 +830,99 @@ TEST(DataWriterTests, UnregisterInstance)
     ASSERT_NE(instance_topic, nullptr);
 
     // Create disabled DataWriters
-    DataWriter* datawriter = publisher->create_datawriter(topic, DATAWRITER_QOS_DEFAULT);
+    datawriter = publisher->create_datawriter(topic, DATAWRITER_QOS_DEFAULT);
     ASSERT_NE(nullptr, datawriter);
-    DataWriter* instance_datawriter = publisher->create_datawriter(instance_topic, DATAWRITER_QOS_DEFAULT);
+    instance_datawriter = publisher->create_datawriter(instance_topic, DATAWRITER_QOS_DEFAULT);
     ASSERT_NE(nullptr, instance_datawriter);
 
+    if (nullptr != keyed_type_support)
+    {
+        *keyed_type_support = instance_type;
+    }
+}
+
+/**
+ * This test checks register_instance API
+ */
+TEST(DataWriterTests, RegisterInstance)
+{
+    // Test parameters
+    InstanceFooType data;
+    data.message("HelloWorld");
+
+    // Create disabled DataWriters
+    DataWriter* datawriter;
+    DataWriter* instance_datawriter;
+    create_writers_for_instance_test(datawriter, instance_datawriter);
+
+    // 1. Calling register_instance in a disable writer returns HANDLE_NIL
+    EXPECT_EQ(HANDLE_NIL, datawriter->register_instance(&data));
+    EXPECT_EQ(HANDLE_NIL, instance_datawriter->register_instance(&data));
+
+    // 2. Calling register_instance in a non keyed topic returns HANDLE_NIL
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, datawriter->enable());
+    EXPECT_EQ(HANDLE_NIL, datawriter->register_instance(&data));
+
+    // 3. Calling register_instance with an invalid sample returns HANDLE_NIL
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->enable());
+    EXPECT_EQ(HANDLE_NIL, instance_datawriter->register_instance(nullptr));
+
+    // 4. Calling register_instance with a valid key returns a valid handle
+    EXPECT_NE(HANDLE_NIL, instance_datawriter->register_instance(&data));
+}
+
+/**
+ * This test checks register_instance_w_timestamp API
+ */
+TEST(DataWriterTests, RegisterInstanceWithTimestamp)
+{
+    // Test parameters
+    InstanceFooType data;
+    data.message("HelloWorld");
+
+    // Create disabled DataWriters
+    DataWriter* datawriter;
+    DataWriter* instance_datawriter;
+    create_writers_for_instance_test(datawriter, instance_datawriter);
+
+    eprosima::fastrtps::Time_t ts{ 0, 1 };
+
+    // 1. Calling register_instance_w_timestamp in a disable writer returns HANDLE_NIL
+    EXPECT_EQ(HANDLE_NIL, datawriter->register_instance_w_timestamp(&data, ts));
+    EXPECT_EQ(HANDLE_NIL, instance_datawriter->register_instance_w_timestamp(&data, ts));
+
+    // 2. Calling register_instance_w_timestamp in a non keyed topic returns HANDLE_NIL
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, datawriter->enable());
+    EXPECT_EQ(HANDLE_NIL, datawriter->register_instance_w_timestamp(&data, ts));
+
+    // 3. Calling register_instance with an invalid sample returns HANDLE_NIL
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->enable());
+    EXPECT_EQ(HANDLE_NIL, instance_datawriter->register_instance_w_timestamp(nullptr, ts));
+
+    // 4. Calling register_instance with an invalid timestamps returns HANDLE_NIL
+    EXPECT_EQ(HANDLE_NIL, instance_datawriter->register_instance_w_timestamp(&data, fastrtps::c_TimeInfinite));
+    EXPECT_EQ(HANDLE_NIL, instance_datawriter->register_instance_w_timestamp(&data, fastrtps::c_TimeInvalid));
+
+    // 5. Calling register_instance with a valid key returns a valid handle
+    EXPECT_NE(HANDLE_NIL, instance_datawriter->register_instance_w_timestamp(&data, ts));
+}
+
+/**
+ * This test checks unregister_instance API
+ */
+TEST(DataWriterTests, UnregisterInstance)
+{
+    // Test parameters
+    InstanceHandle_t handle;
+    InstanceFooType data;
+    data.message("HelloWorld");
+
+    // Create disabled DataWriters
+    TypeSupport instance_type;
+    DataWriter* datawriter;
+    DataWriter* instance_datawriter;
+    create_writers_for_instance_test(datawriter, instance_datawriter, &instance_type);
+
     // 1. Calling unregister_instance in a disable writer returns RETCODE_NOT_ENABLED
     EXPECT_EQ(ReturnCode_t::RETCODE_NOT_ENABLED, datawriter->unregister_instance(&data, handle));
 
@@ -812,12 +944,12 @@ TEST(DataWriterTests, UnregisterInstance)
     EXPECT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET, instance_datawriter->unregister_instance(&data, handle));
 
     // 6. Calling unregister_instance with a valid key returns RETCODE_OK
-    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, c_InstanceHandle_Unknown));
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, HANDLE_NIL));
     EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->unregister_instance(&data, handle));
 
     // 7. Calling unregister_instance with a valid InstanceHandle also returns RETCODE_OK
     data.message("HelloWorld_1");
-    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, c_InstanceHandle_Unknown));
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, HANDLE_NIL));
     instance_type.get_key(&data, &handle);
     EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->unregister_instance(&data, handle));
 
@@ -825,6 +957,70 @@ TEST(DataWriterTests, UnregisterInstance)
     // RETCODE_ERROR, and RETCODE_TIMEOUT (only if HAVE_STRICT_REALTIME has been defined).
 }
 
+/**
+ * This test checks unregister_instance_w_timestamp API
+ */
+TEST(DataWriterTests, UnregisterInstanceWithTimestamp)
+{
+    // Test parameters
+    InstanceHandle_t handle;
+    InstanceFooType data;
+    data.message("HelloWorld");
+
+    // Create disabled DataWriters
+    TypeSupport instance_type;
+    DataWriter* datawriter;
+    DataWriter* instance_datawriter;
+    create_writers_for_instance_test(datawriter, instance_datawriter, &instance_type);
+
+    eprosima::fastrtps::Time_t ts{ 0, 1 };
+
+    // 1. Calling unregister_instance in a disable writer returns RETCODE_NOT_ENABLED
+    EXPECT_EQ(ReturnCode_t::RETCODE_NOT_ENABLED, datawriter->unregister_instance_w_timestamp(&data, handle, ts));
+
+    // 2. Calling unregister_instance in a non keyed topic returns RETCODE_PRECONDITION_NOT MET
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, datawriter->enable());
+    EXPECT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET,
+            datawriter->unregister_instance_w_timestamp(&data, handle, ts));
+
+    // 3. Calling unregister_instance with an invalid sample returns RETCODE_BAD_PARAMETER
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->enable());
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER,
+            instance_datawriter->unregister_instance_w_timestamp(nullptr, handle, ts));
+
+#if !defined(NDEBUG)
+    // 4. Calling unregister_instance with an inconsistent handle returns RETCODE_PRECONDITION_NOT_MET
+    EXPECT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET, instance_datawriter->unregister_instance_w_timestamp(&data,
+            datawriter->get_instance_handle(), ts));
+#endif // NDEBUG
+
+    // 5. Calling unregister_instance with a key not yet registered returns RETCODE_PRECONDITION_NOT_MET
+    EXPECT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET,
+            instance_datawriter->unregister_instance_w_timestamp(&data, handle, ts));
+
+    // 6. Calling unregister_instance with a valid key returns RETCODE_OK
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write_w_timestamp(&data, HANDLE_NIL, ts));
+    EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->unregister_instance_w_timestamp(&data, handle, ts));
+
+    // 7. Calling unregister_instance with a valid InstanceHandle also returns RETCODE_OK
+    data.message("HelloWorld_1");
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write_w_timestamp(&data, HANDLE_NIL, ts));
+    instance_type.get_key(&data, &handle);
+    EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->unregister_instance_w_timestamp(&data, handle, ts));
+
+    // 8. Check invalid timestamps
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write_w_timestamp(&data, HANDLE_NIL, ts));
+    ts = eprosima::fastrtps::c_TimeInfinite;
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER,
+            instance_datawriter->unregister_instance_w_timestamp(&data, handle, ts));
+    ts = eprosima::fastrtps::c_TimeInvalid;
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER,
+            instance_datawriter->unregister_instance_w_timestamp(&data, handle, ts));
+
+    // TODO(jlbueno) There are other possible errors sending the unregister message: RETCODE_OUT_OF_RESOURCES,
+    // RETCODE_ERROR, and RETCODE_TIMEOUT (only if HAVE_STRICT_REALTIME has been defined).
+}
+
 /**
  * This test checks dispose API
  */
@@ -835,34 +1031,11 @@ TEST(DataWriterTests, Dispose)
     InstanceFooType data;
     data.message("HelloWorld");
 
-    // Create participant
-    DomainParticipant* participant =
-            DomainParticipantFactory::get_instance()->create_participant(0, PARTICIPANT_QOS_DEFAULT);
-    ASSERT_NE(nullptr, participant);
-
-    // Create publisher
-    PublisherQos pqos = PUBLISHER_QOS_DEFAULT;
-    pqos.entity_factory().autoenable_created_entities = false;
-    Publisher* publisher = participant->create_publisher(pqos);
-    ASSERT_NE(nullptr, publisher);
-
-    // Register types and topics
-    TypeSupport type(new TopicDataTypeMock());
-    type.register_type(participant);
-    TypeSupport instance_type(new InstanceTopicDataTypeMock());
-    instance_type.register_type(participant);
-
-    Topic* topic = participant->create_topic("footopic", type.get_type_name(), TOPIC_QOS_DEFAULT);
-    ASSERT_NE(topic, nullptr);
-    Topic* instance_topic = participant->create_topic("instancefootopic", instance_type.get_type_name(),
-                    TOPIC_QOS_DEFAULT);
-    ASSERT_NE(instance_topic, nullptr);
-
     // Create disabled DataWriters
-    DataWriter* datawriter = publisher->create_datawriter(topic, DATAWRITER_QOS_DEFAULT);
-    ASSERT_NE(nullptr, datawriter);
-    DataWriter* instance_datawriter = publisher->create_datawriter(instance_topic, DATAWRITER_QOS_DEFAULT);
-    ASSERT_NE(nullptr, instance_datawriter);
+    TypeSupport instance_type;
+    DataWriter* datawriter;
+    DataWriter* instance_datawriter;
+    create_writers_for_instance_test(datawriter, instance_datawriter, &instance_type);
 
     // 1. Calling dispose in a disable writer returns RETCODE_NOT_ENABLED
     EXPECT_EQ(ReturnCode_t::RETCODE_NOT_ENABLED, datawriter->dispose(&data, handle));
@@ -885,12 +1058,12 @@ TEST(DataWriterTests, Dispose)
     EXPECT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET, instance_datawriter->dispose(&data, handle));
 
     // 6. Calling dispose with a valid key returns RETCODE_OK
-    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, c_InstanceHandle_Unknown));
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, HANDLE_NIL));
     EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->dispose(&data, handle));
 
     // 7. Calling dispose with a valid InstanceHandle also returns RETCODE_OK
     data.message("HelloWorld_1");
-    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, c_InstanceHandle_Unknown));
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, HANDLE_NIL));
     instance_type.get_key(&data, &handle);
     EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->dispose(&data, handle));
 
@@ -898,6 +1071,119 @@ TEST(DataWriterTests, Dispose)
     // RETCODE_ERROR, and RETCODE_TIMEOUT (only if HAVE_STRICT_REALTIME has been defined).
 }
 
+/**
+ * This test checks dispose_w_timestamp API
+ */
+TEST(DataWriterTests, DisposeWithTimestamp)
+{
+    // Test parameters
+    InstanceHandle_t handle;
+    InstanceFooType data;
+    data.message("HelloWorld");
+
+    // Create disabled DataWriters
+    TypeSupport instance_type;
+    DataWriter* datawriter;
+    DataWriter* instance_datawriter;
+    create_writers_for_instance_test(datawriter, instance_datawriter, &instance_type);
+
+    eprosima::fastrtps::Time_t ts{ 0, 1 };
+
+    // 1. Calling dispose in a disable writer returns RETCODE_NOT_ENABLED
+    EXPECT_EQ(ReturnCode_t::RETCODE_NOT_ENABLED, datawriter->dispose_w_timestamp(&data, handle, ts));
+
+    // 2. Calling dispose in a non keyed topic returns RETCODE_PRECONDITION_NOT MET
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, datawriter->enable());
+    EXPECT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET, datawriter->dispose_w_timestamp(&data, handle, ts));
+
+    // 3. Calling dispose with an invalid sample returns RETCODE_BAD_PARAMETER
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->enable());
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, instance_datawriter->dispose_w_timestamp(nullptr, handle, ts));
+
+#if !defined(NDEBUG)
+    // 4. Calling dispose with an inconsistent handle returns RETCODE_PRECONDITION_NOT_MET
+    EXPECT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET, instance_datawriter->dispose_w_timestamp(&data,
+            datawriter->get_instance_handle(), ts));
+#endif // NDEBUG
+
+    // 5. Calling dispose with a key not yet registered returns RETCODE_PRECONDITION_NOT_MET
+    EXPECT_EQ(ReturnCode_t::RETCODE_PRECONDITION_NOT_MET, instance_datawriter->dispose_w_timestamp(&data, handle, ts));
+
+    // 6. Calling dispose with a valid key returns RETCODE_OK
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write_w_timestamp(&data, HANDLE_NIL, ts));
+    EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->dispose_w_timestamp(&data, handle, ts));
+
+    // 7. Calling dispose with a valid InstanceHandle also returns RETCODE_OK
+    data.message("HelloWorld_1");
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write_w_timestamp(&data, HANDLE_NIL, ts));
+    instance_type.get_key(&data, &handle);
+    EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->dispose_w_timestamp(&data, handle, ts));
+
+    // 8. Check invalid timestamps
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write_w_timestamp(&data, HANDLE_NIL, ts));
+    ts = eprosima::fastrtps::c_TimeInfinite;
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, instance_datawriter->dispose_w_timestamp(&data, handle, ts));
+    ts = eprosima::fastrtps::c_TimeInvalid;
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, instance_datawriter->dispose_w_timestamp(&data, handle, ts));
+
+    // TODO(jlbueno) There are other possible errors sending the dispose message: RETCODE_OUT_OF_RESOURCES,
+    // RETCODE_ERROR, and RETCODE_TIMEOUT (only if HAVE_STRICT_REALTIME has been defined).
+}
+
+/**
+ * This test checks get_key_value API
+ */
+TEST(DataWriterTests, GetKeyValue)
+{
+    // Test parameters
+    InstanceFooType data;
+    InstanceHandle_t wrong_handle;
+    wrong_handle.value[0] = 0xee;
+    InstanceHandle_t valid_handle;
+    InstanceFooType valid_data;
+    valid_data.message("HelloWorld");
+
+    // Create disabled DataWriters
+    DataWriter* datawriter;
+    DataWriter* instance_datawriter;
+    create_writers_for_instance_test(datawriter, instance_datawriter);
+
+    // 1. Check nullptr on key_holder
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, datawriter->get_key_value(nullptr, wrong_handle));
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, instance_datawriter->get_key_value(nullptr, wrong_handle));
+
+    // 2. Check HANDLE_NIL
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, datawriter->get_key_value(&data, HANDLE_NIL));
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, instance_datawriter->get_key_value(&data, HANDLE_NIL));
+
+    // 3. Check type should have keys, and writer should be enabled
+    EXPECT_EQ(ReturnCode_t::RETCODE_ILLEGAL_OPERATION, datawriter->get_key_value(&data, wrong_handle));
+    EXPECT_EQ(ReturnCode_t::RETCODE_NOT_ENABLED, instance_datawriter->get_key_value(&data, wrong_handle));
+
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, datawriter->enable());
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->enable());
+
+    // 4. Calling get_key_value with a key not yet registered returns RETCODE_BAD_PARAMETER
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, instance_datawriter->get_key_value(&data, wrong_handle));
+
+    // 5. Calling get_key_value on a registered instance should work.
+    valid_handle = instance_datawriter->register_instance(&valid_data);
+    EXPECT_NE(HANDLE_NIL, valid_handle);
+    EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->get_key_value(&data, valid_handle));
+
+    // 6. Calling get_key_value on an unregistered instance should return RETCODE_BAD_PARAMETER.
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->unregister_instance(&valid_data, valid_handle));
+    EXPECT_EQ(ReturnCode_t::RETCODE_BAD_PARAMETER, instance_datawriter->get_key_value(&data, valid_handle));
+
+    // 7. Calling get_key_value with a valid instance should work
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&valid_data, HANDLE_NIL));
+    EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->get_key_value(&data, valid_handle));
+
+    // 8. Calling get_key_value on a disposed instance should work.
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->dispose(&valid_data, valid_handle));
+    EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->get_key_value(&data, valid_handle));
+}
+
 struct LoanableType
 {
     static constexpr uint32_t initialization_value()
@@ -1160,7 +1446,7 @@ class DataWriterImplTest : public DataWriterImpl
 {
 public:
 
-    fastrtps::PublisherHistory* get_history()
+    DataWriterHistory* get_history()
     {
         return &history_;
     }
@@ -1233,12 +1519,12 @@ TEST(DataWriterTests, InstanceWaitForAcknowledgement)
     ASSERT_NE(nullptr, datawriter_impl);
     DataWriterImplTest* datawriter_impl_test = static_cast<DataWriterImplTest*>(datawriter_impl);
     ASSERT_NE(nullptr, datawriter_impl_test);
-    fastrtps::PublisherHistory* history = datawriter_impl_test->get_history();
+    auto history = datawriter_impl_test->get_history();
 
-    // 5. Calling wait_for_acknowledgments in a keyed topic with c_InstanceHandle_Unknown returns
+    // 5. Calling wait_for_acknowledgments in a keyed topic with HANDLE_NIL returns
     // RETCODE_OK
     EXPECT_CALL(*history, wait_for_acknowledgement_last_change(_, _, _)).WillOnce(testing::Return(true));
-    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, c_InstanceHandle_Unknown));
+    ASSERT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->write(&data, HANDLE_NIL));
     EXPECT_EQ(ReturnCode_t::RETCODE_OK, instance_datawriter->wait_for_acknowledgments(&data, handle,
             max_wait));
 
@@ -1310,12 +1596,8 @@ public:
  * This test checks that the DataWriter methods defined in the standard not yet implemented in FastDDS return
  * ReturnCode_t::RETCODE_UNSUPPORTED. The following methods are checked:
  * 1. get_matched_subscription_data
- * 2. write_w_timestamp
- * 3. register_instance_w_timestamp
- * 4. unregister_instance_w_timestamp
- * 5. get_matched_subscriptions
- * 6. get_key_value
- * 7. lookup_instance
+ * 2. get_matched_subscriptions
+ * 3. lookup_instance
  */
 TEST_F(DataWriterUnsupportedTests, UnsupportedDataWriterMethods)
 {
@@ -1341,40 +1623,13 @@ TEST_F(DataWriterUnsupportedTests, UnsupportedDataWriterMethods)
         ReturnCode_t::RETCODE_UNSUPPORTED,
         data_writer->get_matched_subscription_data(subscription_data, subscription_handle));
 
-    {
-        InstanceHandle_t handle;
-        fastrtps::rtps::Time_t timestamp;
-        EXPECT_EQ(
-            ReturnCode_t::RETCODE_UNSUPPORTED,
-            data_writer->write_w_timestamp(nullptr /* data */, handle, timestamp));
-    }
-
-    {
-        fastrtps::rtps::Time_t timestamp;
-        EXPECT_EQ(
-            HANDLE_NIL,
-            data_writer->register_instance_w_timestamp(nullptr /* instance */, timestamp));
-    }
-
-    {
-        InstanceHandle_t handle;
-        fastrtps::rtps::Time_t timestamp;
-        EXPECT_EQ(
-            ReturnCode_t::RETCODE_UNSUPPORTED,
-            data_writer->unregister_instance_w_timestamp(nullptr /* instance */, handle, timestamp));
-    }
-
-
-    std::vector<fastrtps::rtps::InstanceHandle_t*> subscription_handles;
+    std::vector<InstanceHandle_t> subscription_handles;
     EXPECT_EQ(ReturnCode_t::RETCODE_UNSUPPORTED, data_writer->get_matched_subscriptions(subscription_handles));
 
-    fastrtps::rtps::InstanceHandle_t key_handle;
-    EXPECT_EQ(ReturnCode_t::RETCODE_UNSUPPORTED, data_writer->get_key_value(nullptr /* key_holder */, key_handle));
-
     EXPECT_EQ(HANDLE_NIL, data_writer->lookup_instance(nullptr /* instance */));
 
-    // Expected logWarnings: register_instance_w_timestamp, lookup_instance
-    HELPER_WaitForEntries(2);
+    // Expected logWarnings: lookup_instance
+    HELPER_WaitForEntries(1);
 
     ASSERT_EQ(publisher->delete_datawriter(data_writer), ReturnCode_t::RETCODE_OK);
     ASSERT_EQ(participant->delete_publisher(publisher), ReturnCode_t::RETCODE_OK);
