@@ -121,6 +121,7 @@ TEST(WriterProxyTests, MissingChangesUpdate)
     // and lastSN.value < firstSN.value
     bool assert_liveliness = false;
     uint32_t heartbeat_count = 1;
+    int32_t current_sample_lost = 0;
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
     wproxy.process_heartbeat(
         heartbeat_count++,
@@ -129,7 +130,8 @@ TEST(WriterProxyTests, MissingChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness );
+        assert_liveliness,
+        current_sample_lost);
 
     SequenceNumberSet_t t2 (SequenceNumber_t(0, 1));
     t2.add(SequenceNumber_t(0, 1));
@@ -137,6 +139,7 @@ TEST(WriterProxyTests, MissingChangesUpdate)
     ASSERT_EQ(SequenceNumber_t(), wproxy.available_changes_max());
     ASSERT_EQ(0u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 1)));
     ASSERT_EQ(1u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 2)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 3. Simulate reception of a HEARTBEAT after two more samples are added to the writer's history
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
@@ -147,7 +150,8 @@ TEST(WriterProxyTests, MissingChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     t2.add(SequenceNumber_t(0, 2));
     t2.add(SequenceNumber_t(0, 3));
@@ -157,6 +161,7 @@ TEST(WriterProxyTests, MissingChangesUpdate)
     ASSERT_EQ(1u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 2)));
     ASSERT_EQ(2u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 3)));
     ASSERT_EQ(3u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 4)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 4. Simulate reception of a DATA(6).
     wproxy.received_change_set(SequenceNumber_t(0, 6));
@@ -170,6 +175,7 @@ TEST(WriterProxyTests, MissingChangesUpdate)
     ASSERT_EQ(4u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 5)));
     ASSERT_EQ(5u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 6)));
     ASSERT_EQ(5u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 7)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 5. Simulate reception of a HEARTBEAT(1,6)
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
@@ -180,13 +186,15 @@ TEST(WriterProxyTests, MissingChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     ASSERT_THAT(t2, wproxy.missing_changes());
     ASSERT_EQ(SequenceNumber_t(), wproxy.available_changes_max());
     ASSERT_EQ(4u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 5)));
     ASSERT_EQ(5u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 6)));
     ASSERT_EQ(5u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 7)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 5. Simulate reception of a HEARTBEAT(1,7)
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
@@ -197,7 +205,8 @@ TEST(WriterProxyTests, MissingChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     t2.add(SequenceNumber_t(0, 7));
     ASSERT_THAT(t2, wproxy.missing_changes());
@@ -206,6 +215,7 @@ TEST(WriterProxyTests, MissingChangesUpdate)
     ASSERT_EQ(5u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 6)));
     ASSERT_EQ(5u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 7)));
     ASSERT_EQ(6u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 8)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 6. Simulate reception of all missing DATA
     wproxy.received_change_set(SequenceNumber_t(0, 1));
@@ -220,6 +230,7 @@ TEST(WriterProxyTests, MissingChangesUpdate)
     ASSERT_EQ(SequenceNumber_t(0, 6), wproxy.available_changes_max());
     ASSERT_EQ(0u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 7)));
     ASSERT_EQ(1u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 8)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 7. Simulate reception of a faulty HEARTBEAT with a lower last sequence number (4)
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
@@ -230,10 +241,12 @@ TEST(WriterProxyTests, MissingChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness );
+        assert_liveliness,
+        current_sample_lost);
 
     ASSERT_THAT(t6, wproxy.missing_changes());
     ASSERT_EQ(SequenceNumber_t(0, 6), wproxy.available_changes_max());
+    ASSERT_EQ(0, current_sample_lost);
 
     // 8. Simulate reception of DATA(8) and DATA(10)
     wproxy.received_change_set(SequenceNumber_t(0, 8));
@@ -248,6 +261,7 @@ TEST(WriterProxyTests, MissingChangesUpdate)
     ASSERT_EQ(4u, wproxy.number_of_changes_from_writer());
     ASSERT_EQ(1u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 9)));
     ASSERT_EQ(2u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 11)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 9. Simulate reception of HEARTBEAT(1,10)
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
@@ -258,13 +272,30 @@ TEST(WriterProxyTests, MissingChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness );
+        assert_liveliness,
+        current_sample_lost);
 
     t6.add(SequenceNumber_t(0, 6));
     t6.add(SequenceNumber_t(0, 7));
     t6.add(SequenceNumber_t(0, 9));
     ASSERT_THAT(t6, wproxy.missing_changes());
     ASSERT_EQ(SequenceNumber_t(0, 6), wproxy.available_changes_max());
+    ASSERT_EQ(0, current_sample_lost);
+
+    // 10. Simulate reception of HEARTBEAT(11,0)
+    EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
+    wproxy.process_heartbeat(
+        heartbeat_count++,
+        SequenceNumber_t( 0, 11 ),
+        SequenceNumber_t( 0, 0 ),
+        false,
+        false,
+        false,
+        assert_liveliness,
+        current_sample_lost);
+
+    ASSERT_EQ(SequenceNumber_t(0, 10), wproxy.available_changes_max());
+    ASSERT_EQ(2, current_sample_lost);
 
 }
 
@@ -281,6 +312,7 @@ TEST(WriterProxyTests, LostChangesUpdate)
     // 1. Simulate reception of a HEARTBEAT(3,3)
     uint32_t heartbeat_count = 1;
     bool assert_liveliness = false;
+    int32_t current_sample_lost = 0;
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
     wproxy.process_heartbeat(
         heartbeat_count++,
@@ -289,7 +321,8 @@ TEST(WriterProxyTests, LostChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     SequenceNumberSet_t t1(SequenceNumber_t(0, 3));
     t1.add(SequenceNumber_t(0, 3));
@@ -297,6 +330,7 @@ TEST(WriterProxyTests, LostChangesUpdate)
     ASSERT_EQ(SequenceNumber_t(0, 2), wproxy.available_changes_max());
     ASSERT_EQ(1u, wproxy.number_of_changes_from_writer());
     ASSERT_EQ(1u, wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 4)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 2. Simulate reception of a DATA(5) follow by a HEARTBEAT(5,5)
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
@@ -308,12 +342,14 @@ TEST(WriterProxyTests, LostChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     ASSERT_THAT( SequenceNumberSet_t(SequenceNumber_t( 0, 6)), wproxy.missing_changes());
     ASSERT_EQ( SequenceNumber_t( 0, 5 ), wproxy.available_changes_max());
     ASSERT_EQ( 0u, wproxy.number_of_changes_from_writer());
     ASSERT_EQ( 0u, wproxy.unknown_missing_changes_up_to( SequenceNumber_t( 0, 5 )));
+    ASSERT_EQ(2, current_sample_lost);
 
     // 3. Simulate reception of a faulty HEARTBEAT with a lower first sequence number (4)
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
@@ -324,11 +360,13 @@ TEST(WriterProxyTests, LostChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     ASSERT_THAT(SequenceNumberSet_t( SequenceNumber_t(0, 6)), wproxy.missing_changes());
     ASSERT_EQ(SequenceNumber_t(0, 5), wproxy.available_changes_max());
     ASSERT_EQ(0u, wproxy.unknown_missing_changes_up_to( SequenceNumber_t(0, 5)));
+    ASSERT_EQ(0, current_sample_lost);
 
     // 4. Simulate reception of a DATA(7)
     // According to the RTPS standard, sequence number 5 would be missing and 6 would be unknown,
@@ -351,12 +389,14 @@ TEST(WriterProxyTests, LostChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     SequenceNumberSet_t t5(SequenceNumber_t(0, 8));
     t5.add(SequenceNumber_t(0, 8));
     ASSERT_THAT(t5, wproxy.missing_changes());
     ASSERT_EQ(SequenceNumber_t(0, 7), wproxy.available_changes_max());
+    ASSERT_EQ(1, current_sample_lost);
 
     // 6. Simulate reception of a HEARTBEAT(10,10)
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
@@ -367,12 +407,14 @@ TEST(WriterProxyTests, LostChangesUpdate)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     SequenceNumberSet_t t6(SequenceNumber_t(0, 10));
     t6.add(SequenceNumber_t(0, 10));
     ASSERT_THAT(t6, wproxy.missing_changes());
     ASSERT_EQ(SequenceNumber_t(0, 9), wproxy.available_changes_max());
+    ASSERT_EQ(2, current_sample_lost);
 }
 
 TEST(WriterProxyTests, ReceivedChangeSet)
@@ -476,6 +518,7 @@ TEST(WriterProxyTests, ReceivedChangeSet)
     // Sequence number 6 should be RECEIVED
     bool assert_liveliness = false;
     uint32_t heartbeat_count = 1;
+    int32_t current_sample_lost = 0;
     EXPECT_CALL(*wproxy.heartbeat_response_, restart_timer()).Times(1u);
     wproxy.process_heartbeat(
         heartbeat_count++,
@@ -484,12 +527,14 @@ TEST(WriterProxyTests, ReceivedChangeSet)
         false,
         false,
         false,
-        assert_liveliness);
+        assert_liveliness,
+        current_sample_lost);
 
     ASSERT_THAT(t4, wproxy.missing_changes());
     ASSERT_EQ(wproxy.are_there_missing_changes(), true);
     ASSERT_EQ(wproxy.number_of_changes_from_writer(), 3u);
     ASSERT_EQ(wproxy.unknown_missing_changes_up_to(SequenceNumber_t(0, 7)), 2u);
+    ASSERT_EQ(0, current_sample_lost);
 
     // 6. Writer proxy receives DATA with sequence number 8
     // Sequence number 4 should be MISSING
