@@ -18,6 +18,7 @@
 #include <asio.hpp>
 #include <gtest/gtest.h>
 #include <MockReceiverResource.h>
+#include "mock/MockTCPChannelResource.h"
 #include "mock/MockTCPv4Transport.h"
 #include <fastdds/dds/log/Log.hpp>
 #include <fastrtps/transport/TCPv4TransportDescriptor.h>
@@ -1464,6 +1465,49 @@ TEST_F(TCPv4Tests, receive_unordered_data)
     EXPECT_TRUE(uut.CloseInputChannel(input_locator));
 }
 
+// This test verifies that disabling a TCPChannelResource in the middle of a Receive call (invoked in
+// perform_listen_operation) does not result in a hungup state [13721].
+TEST_F(TCPv4Tests, header_read_interrumption)
+{
+    eprosima::fastdds::dds::Log::SetVerbosity(eprosima::fastdds::dds::Log::Kind::Info);
+    std::regex filter("RTCP(?!_SEQ)");
+    eprosima::fastdds::dds::Log::SetCategoryFilter(filter);
+
+    TCPv4TransportDescriptor test_descriptor;
+    test_descriptor.add_listener_port(g_default_port);
+    TCPv4Transport transportUnderTest(test_descriptor);
+    transportUnderTest.init();
+
+    Locator_t locator;
+    locator.kind = LOCATOR_KIND_TCPv4;
+    locator.port = g_default_port;
+    IPLocator::setIPv4(locator, 127, 0, 0, 1);
+    IPLocator::setLogicalPort(locator, 7410);
+
+    std::weak_ptr<eprosima::fastdds::rtps::RTCPMessageManager> rtcp_manager =
+            std::make_shared<eprosima::fastdds::rtps::RTCPMessageManager>(&transportUnderTest);
+    std::shared_ptr<TCPChannelResource> channel = std::make_shared<MockTCPChannelResource>(&transportUnderTest, locator,
+                    32767);
+    octet* buffer = {};
+    uint32_t receive_buffer_capacity = 65500;
+    uint32_t receive_buffer_size = 0;
+
+    // Simulate channel connection
+    channel->connect(nullptr);
+
+    // Simulate channel disconnection after a second
+    std::thread thread([&channel]
+            {
+                std::this_thread::sleep_for(std::chrono::seconds(1));
+                channel->disable();
+            });
+
+    // Start TCP segment reception
+    // Should get stuck in receive_header until channel is disabled
+    transportUnderTest.Receive(rtcp_manager, channel, buffer, receive_buffer_capacity, receive_buffer_size, locator);
+    thread.join();
+}
+
 void TCPv4Tests::HELPER_SetDescriptorDefaults()
 {
     descriptor.add_listener_port(g_default_port);
